/*
	File:		GenericIcons.r

	Contains:	Generic icons used by standard file, finder, alias manager, and apps

				note: in system 6.X, they used to be embedded in the PACK and finder
				note: to use these in a popup menu, you have to patch GetResource
						to convert the id from the 256-512 range to this range

				ICN# are 32x32 and always have a mask
				ics# are 16x16 and always have a mask
				SICN are 12x12 and have a mask only if Finder uses them

				Finder uses entire families of icons
				Standard File LDEF uses the SICNs only

	Copyright:	 1990-1992 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<33>	  5/4/92	SMB		#1027665,<jmg>: Fixed the fonts folder icons.
		<32>	11/19/91	JSM		Update with new model icons from Paulien (gray in PowerBook 100
									color icons a shade too dark, move large Quadra icons down one
									pixel, fix mask in 'ics#' for Quadra 700 which was too short);
									use new #defines for model icons and systemSuitcaseIconResource;
									always use resource statement, not data; clean up formatting of
									lots of icons to show their visual appearance better here.
		<31>	 11/5/91	DTY		Add icons for Fonts Folder.
		<30>	10/28/91	SAM/KSM	Rolled in Regatta file:
										(BG) Updated Eclipse icons.
										(HJR) Update Icons for Eclipse, Tim, Spike, and Asahi.
										Added Icons for Asahi. (HJR) Added machine icons for Eclipse,
											Spike, and Tim.
		<29>	 3/28/91	DTY		Shade the handles a little better.
		<28>	 3/28/91	DTY		Make the handles in the trash icons gray for Paul.
		<27>	 3/21/91	DTY		Change the trash icon masks so you cant see through the handle.
		<26>	 3/13/91	dba		bbm, #83173: realign stationery icon
		<25>	 2/20/91	csd		dty, #GangOf5: Updated the Control Panels folder icon.
		<24>	  2/8/91	dba		dty, #82470: add an 'SICN' for drop folders and fix the 'SICN'
									for private folder and drop folders to exactly match the normal
									folder icon in size and position
		<23>	 1/17/91	DC		csd - updated Generic extenstion (-16415) generic stationary and
									the extension folder
		<22>	 1/17/91	csd		& JDR: Removed the mailbox and letter SICNs since they arent
									part of 7.0
		<21>	12/27/90	csd		Added the Clipboard File icon (-16509) and updated the generic
									query document icon (-16506, genericQueryDocumentIconResource)
		<20>	12/27/90	csd		Updated these icons: -16494, -16484, -16482, -16415, -3998
									(floppyIconResource), -3996 (genericApplicationIconResource),
									-3993 (trashIconResource), -3991
									(genericDeskAccessoryIconResource), -3989
									(genericEditionFileIconResource), -3985
									(genericStationeryIconResource), -3984 (fullTrashIconResource),
									-3983 (systemFolderIconResource), -3982
									(appleMenuFolderIconResource), -3981
									(startupFolderIconResource), -3976
									(controlPanelFolderIconResource), 3; also removed the System
									Folder SICN
		<19>	11/23/90	csd		& gbm; Updated the drop folder icon suite
		<17>	11/19/90	DC		ngk - Changed identifier names for generic icons to conform to new
									public names
		<16>	 11/1/90	csd		Add the final Elsie and Erickson icon suites.
		<15>	10/31/90	csd		Added the generic query document, unopenable folder, generic DA,
									generic suitcase, and generic Mover object icons.
		<14>	10/18/90	ngk		Added SICNs for DAs and stationery.
		<13>	 8/30/90	ngk		moved constants into IconUtilsPrivTypes.r
		<12>	 8/15/90	csd		Update the generic extension icon and added the rest of the
									suite.
		<11>	 8/14/90	csd		Updated the mounted folder icons.
		<10>	 7/30/90	JSM		Add genericPreferencesIconResource, explicilty state limits on
									ids.
		 <9>	  7/2/90	csd		Updated the System File icon.
		 <8>	  7/2/90	csd		Fix the 3-slot and 6-slot Mac icons to align with the mask.
									Added a mask to the edition SICN.
		 <7>	  6/8/90	ngk		Added genericFileServerIconResource
		 <6>	  6/3/90	csd		Added icon family -16494 for the System file icon.
		 <5>	 4/23/90	dba		put in SICN 3 for Paul
		 <4>	 4/11/90	dba		added a bunch more icons; fixed drop folder icon for Paul
		 <3>	 4/10/90	PP		Add 'SICN' mask for Trash, Floppy and Hard disk.
		 							Include "Types.r".
		 <2>	 4/10/90	ngk		fix comments
		 <1>	 4/10/90	ngk		moved icons out of StandardFile3.r into here

	To Do:
	
		add small icons for:  IIcx/IIci, II/IIx/IIfx, 
*/

#include "Types.r"
#include $$Shell("IntRIncludes")"IconUtilsPrivTypes.r"

//__________________________________________________________________________________________________
// generic document

resource 'ICN#' (genericDocumentIconResource, purgeable)
	{ {
	$"1FFFFC00 10000600 10000500 10000480"
	$"10000440 10000420 100007F0 10000010"
	$"10000010 10000010 10000010 10000010"
	$"10000010 10000010 10000010 10000010"
	$"10000010 10000010 10000010 10000010"
	$"10000010 10000010 10000010 10000010"
	$"10000010 10000010 10000010 10000010"
	$"10000010 10000010 10000010 1FFFFFF0";

	$"1FFFFC00 1FFFFE00 1FFFFF00 1FFFFF80"
	$"1FFFFFC0 1FFFFFE0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	} };

resource 'ics#' (genericDocumentIconResource, purgeable)
	{ {
	$"7FE0 4030 4028 403C 4004 4004 4004 4004"
	$"4004 4004 4004 4004 4004 4004 4004 7FFC";

	$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
	$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	} };

resource 'icl8' (genericDocumentIconResource, purgeable)
	{
	$"000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFF000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2BFF0000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2BFF00000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2B2BFF000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2B2B2BFF0000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFFFFFFFFFFF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	};

resource 'icl4' (genericDocumentIconResource, purgeable)
	{
	$"000FFFFFFFFFFFFFFFFFFF0000000000"
	$"000F0C0C0C0C0C0C0C0C0FF000000000"
	$"000FC0C0C0C0C0C0C0C0CFCF00000000"
	$"000F0C0C0C0C0C0C0C0C0FCCF0000000"
	$"000FC0C0C0C0C0C0C0C0CFCCCF000000"
	$"000F0C0C0C0C0C0C0C0C0FCCCCF00000"
	$"000FC0C0C0C0C0C0C0C0CFFFFFFF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000FFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics8' (genericDocumentIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFF0000000000"
	$"00FFF5F5F5F5F5F5F5F5FFFF00000000"
	$"00FFF5F5F5F5F5F5F5F5FF2BFF000000"
	$"00FFF5F5F5F5F5F5F5F5FFFFFFFF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics4' (genericDocumentIconResource, purgeable)
	{
	$"0FFFFFFFFFF00000"
	$"0F0C0C0C0CFF0000"
	$"0FC0C0C0C0FCF000"
	$"0F0C0C0C0CFFFF00"
	$"0FC0C0C0C0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0FFFFFFFFFFFFF00"
	};

resource 'SICN' (genericDocumentIconResource, purgeable)
	{ {
	$"0000 0000 0000 0FC0 0860 0870 0810 0810"
	$"0810 0810 0810 0810 0FF0 0000 0000 0000";

	$"0000 0000 0000 0FC0 0FE0 0FF0 0FF0 0FF0"
	$"0FF0 0FF0 0FF0 0FF0 0FF0 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic application

resource 'ICN#' (genericApplicationIconResource, purgeable)
	{ {
	$"00010000 00028000 00044000 00082000"
	$"00101000 00200800 00400400 00800200"
	$"01000100 02000080 04000040 08000020"
	$"10000010 20000008 40003F04 80004082"
	$"40008041 20013022 1001C814 080E7F8F"
	$"04023007 02010007 01008007 00806007"
	$"00401FE7 0020021F 00100407 00080800"
	$"00041000 00022000 00014000 00008000";

	$"00010000 00038000 0007C000 000FE000"
	$"001FF000 003FF800 007FFC00 00FFFE00"
	$"01FFFF00 03FFFF80 07FFFFC0 0FFFFFE0"
	$"1FFFFFF0 3FFFFFF8 7FFFFFFC FFFFFFFE"
	$"7FFFFFFF 3FFFFFFE 1FFFFFFC 0FFFFFFF"
	$"07FFFFFF 03FFFFFF 01FFFFFF 00FFFFFF"
	$"007FFFFF 003FFE1F 001FFC07 000FF800"
	$"0007F000 0003E000 0001C000 00008000"
	} };

resource 'ics#' (genericApplicationIconResource, purgeable)
	{ {
	$"0000 0100 0280 0440 0820 1010 2008 4064"
	$"8092 414E 26F6 1106 08FE 0446 0280 0100";

	$"0000 0100 0380 07C0 0FE0 1FF0 3FF8 7FFC"
	$"FFFE 7FFE 3FFE 1FFE 0FFE 07C6 0380 0100"
	} };

resource 'icl8' (genericApplicationIconResource, purgeable)
	{
	$"000000000000000000000000000000FF00000000000000000000000000000000"
	$"0000000000000000000000000000FFF5FF000000000000000000000000000000"
	$"00000000000000000000000000FFF5F5F5FF0000000000000000000000000000"
	$"000000000000000000000000FFF5F5F5F5F5FF00000000000000000000000000"
	$"0000000000000000000000FFF5F5F5F5F5F5F5FF000000000000000000000000"
	$"00000000000000000000FFF5F5F5F5F5F5F5F5F5FF0000000000000000000000"
	$"000000000000000000FFF5F5F5F5F5F5F5F5F5F5F5FF00000000000000000000"
	$"0000000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF000000000000000000"
	$"00000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000000000000000"
	$"000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000000000"
	$"0000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF000000000000"
	$"00000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFFFFFFFFFF5F5F5F5F5FF0000"
	$"FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF080808080808FFF5F5F5F5F5FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0808080808080808FFF5F5F5F5F5FF"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5FF0808FFFF080808080808FFF5F5F5FF00"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5FFFFFFF5F5FF080808080808FFF5FF0000"
	$"00000000FFF5F5F5F5F5F5F5FFFFFF0808FFFFFFFFFFFFFFFF080808FFFFFFFF"
	$"0000000000FFF5F5F5F5F5F5F5F5FF080808FFFF080808080808080808FFFFFF"
	$"000000000000FFF5F5F5F5F5F5F5F5FF08080808080808080808080808FFFFFF"
	$"00000000000000FFF5F5F5F5F5F5F5F5FF080808080808080808080808FFFFFF"
	$"0000000000000000FFF5F5F5F5F5F5F5F5FFFF08080808080808080808FFFFFF"
	$"000000000000000000FFF5F5F5F5F5F5F5F5F5FFFFFFFFFFFFFFFF0808FFFFFF"
	$"00000000000000000000FFF5F5F5F5F5F5F5F5F5F5F5FF00000000FFFFFFFFFF"
	$"0000000000000000000000FFF5F5F5F5F5F5F5F5F5FF00000000000000FFFFFF"
	$"000000000000000000000000FFF5F5F5F5F5F5F5FF0000000000000000000000"
	$"00000000000000000000000000FFF5F5F5F5F5FF000000000000000000000000"
	$"0000000000000000000000000000FFF5F5F5FF00000000000000000000000000"
	$"000000000000000000000000000000FFF5FF0000000000000000000000000000"
	$"00000000000000000000000000000000FF000000000000000000000000000000"
	};

resource 'icl4' (genericApplicationIconResource, purgeable)
	{
	$"000000000000000F0000000000000000"
	$"00000000000000FCF000000000000000"
	$"0000000000000FC0CF00000000000000"
	$"000000000000FC0C0CF0000000000000"
	$"00000000000FC0C0C0CF000000000000"
	$"0000000000FC0C0C0C0CF00000000000"
	$"000000000FC0C0C0C0C0CF0000000000"
	$"00000000FC0C0C0C0C0C0CF000000000"
	$"0000000FC0C0C0C0C0C0C0CF00000000"
	$"000000FC0C0C0C0C0C0C0C0CF0000000"
	$"00000FC0C0C0C0C0C0C0C0C0CF000000"
	$"0000FC0C0C0C0C0C0C0C0C0C0CF00000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"0FC0C0C0C0C0C0C0C0FFFFFFC0C0CF00"
	$"FC0C0C0C0C0C0C0C0F020202FC0C0CF0"
	$"0FC0C0C0C0C0C0C0F02020202FC0C0CF"
	$"00FC0C0C0C0C0C0F02FF020202FC0CF0"
	$"000FC0C0C0C0C0CFFFC0F020202FCF00"
	$"0000FC0C0C0CFFF20FFFFFFFF202FFFF"
	$"00000FC0C0C0C0F020FF202020202FFF"
	$"000000FC0C0C0C0F0202020202020FFF"
	$"0000000FC0C0C0C0F020202020202FFF"
	$"00000000FC0C0C0C0FF2020202020FFF"
	$"000000000FC0C0C0C0CFFFFFFFF02FFF"
	$"0000000000FC0C0C0C0C0CF0000FFFFF"
	$"00000000000FC0C0C0C0CF0000000FFF"
	$"000000000000FC0C0C0CF00000000000"
	$"0000000000000FC0C0CF000000000000"
	$"00000000000000FC0CF0000000000000"
	$"000000000000000FCF00000000000000"
	$"0000000000000000F000000000000000"
	};

resource 'ics8' (genericApplicationIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000FF0000000000000000"
	$"000000000000FFF5FF00000000000000"
	$"0000000000FFF5F5F5FF000000000000"
	$"00000000FFF5F5F5F5F5FF0000000000"
	$"000000FFF5F5F5F5F5F5F5FF00000000"
	$"0000FFF5F5F5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5F5F5F5F5FFFFF5F5FF0000"
	$"FFF5F5F5F5F5F5F5FF0808FFF5F5FF00"
	$"00FFF5F5F5F5F5FF08FF0808FFFFFF00"
	$"0000FFF5F5FFFF08FFFFFFFF08FFFF00"
	$"000000FFF5F5F5FF0808080808FFFF00"
	$"00000000FFF5F5F5FFFFFFFFFFFFFF00"
	$"0000000000FFF5F5F5FF000000FFFF00"
	$"000000000000FFF5FF00000000000000"
	$"00000000000000FF0000000000000000"
	};

resource 'ics4' (genericApplicationIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000F00000000"
	$"000000F0F0000000"
	$"00000F0C0F000000"
	$"0000F0C0C0F00000"
	$"000F0C0C0C0F0000"
	$"00F0C0C0C0C0F000"
	$"0F0C0C0C0FFC0F00"
	$"F0C0C0C0F02FC0F0"
	$"0F0C0C0F2F02FFF0"
	$"00F0CFF0FFFF2FF0"
	$"000F0C0F02020FF0"
	$"0000F0C0FFFFFFF0"
	$"00000F0C0F000FF0"
	$"000000F0F0000000"
	$"0000000F00000000"
	};

resource 'SICN' (genericApplicationIconResource, purgeable)
	{ {
	$"0000 0000 0100 0280 0440 0820 10D0 2128"
	$"13D0 0898 0478 0298 0100 0000 0000 0000";

	$"0000 0000 0100 0380 07C0 0FE0 1FF0 3FF8"
	$"1FF0 0FF8 07F8 03F8 0100 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic folder

resource 'ICN#' (genericFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 FFFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (genericFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 8001"
	$"8001 8001 8001 8001 8001 8001 8001 FFFF";

	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (genericFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (genericFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (genericFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (genericFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

resource 'SICN' (genericFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 1E00 21F0 2008 2008"
	$"2008 2008 2008 2008 3FF8 0000 0000 0000";

	$"0000 0000 0000 0000 1E00 3FF0 3FF8 3FF8"
	$"3FF8 3FF8 3FF8 3FF8 3FF8 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic open folder

resource 'SICN' (openFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 1E00 21FC 2002 FFE2 8012"
	$"4012 400A 200A 2006 1FFE 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// no privileges folder

resource 'ICN#' (privateFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 07F00000 08080000"
	$"10040000 2002FE00 7FFF83FC 80008202"
	$"80008202 80008202 80008202 80008202"
	$"8001FF02 80010102 80017D02 80014502"
	$"80015502 80016D02 80012902 8001EF02"
	$"8000AA02 80009202 80008202 80008202"
	$"80008202 80008202 FFFF83FE 0000FE00";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 07F00000 0FF80000"
	$"1FFC0000 3FFEFE00 7FFFFFFC FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE 0000FE00"
	} };

resource 'ics#' (privateFolderIconResource, purgeable)
	{ {
	$"0000 0000 3C00 42F0 FF9E 8091 8091 81F9"
	$"8109 8169 8169 81F9 8091 8091 FF9F 00F0";

	$"0000 0000 3C00 7EF0 FFFE FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 00F0"
	} };

resource 'icl8' (privateFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF00FFFFFFFFFFFFFF000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFF169F169F16FFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF9F169F169FFF2A2A2A2A2A2A2AFF00"
	$"FF000000000000000000000000000000FF169F169F16FF00000000000000FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF169F169F16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FFFFFFFFFFFFFFFFFF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFF01010101010101FF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FF01FFFFFFFFFF01FF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFF01FF169F16FF01FF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FF01FF9FFF9FFF01FF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFF01FFFF01FFFF01FF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FF0101FF01FF0101FF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFFFFFFFF01FFFFFFFF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF16FF01FF16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F16FF169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF169F169F16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF169F169F16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF169F169F16FFFFFFFFFFFFFFFFFF00"
	$"00000000000000000000000000000000FFFFFFFFFFFFFF000000000000000000"
	};

resource 'icl4' (privateFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF0FFFFFFF000000000"
	$"0FCCCCCCCCCCCCCFFDDDDDFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCFDDDDDFCCCCCCCF0"
	$"F000000000000000FDDDDDF0000000F0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDDDDDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0FFFFFFFFF0C0C0CF0"
	$"F0C0C0C0C0C0C0CF0000000FC0C0CCF0"
	$"FC0C0C0C0C0C0C0F0FFFFF0F0C0C0CF0"
	$"F0C0C0C0C0C0C0CF0FDDDF0FC0C0CCF0"
	$"FC0C0C0C0C0C0C0F0FDFDF0F0C0C0CF0"
	$"F0C0C0C0C0C0C0CF0FF0FF0FC0C0CCF0"
	$"FC0C0C0C0C0C0C0F00F0F00F0C0C0CF0"
	$"F0C0C0C0C0C0C0CFFFF0FFFFC0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDF0FDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDFDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDDDDDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDDDDDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFDDDDDFFFFFFFFF0"
	$"0000000000000000FFFFFFF000000000"
	};

resource 'ics8' (privateFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF00FFFFFFFF00000000"
	$"FF2A2A2A2A2A2AFFFF9F16FFFFFFFF00"
	$"FF002A002A002A00FF169FFF2A0054FF"
	$"FF2A002A002A002AFF9F16FF002A54FF"
	$"FF002A002A002AFFFFFFFFFFFF0054FF"
	$"FF2A002A002A00FF01010101FF2A54FF"
	$"FF002A002A002AFF01FFFF01FF0054FF"
	$"FF2A002A002A00FF01FFFF01FF2A54FF"
	$"FF002A002A002AFFFFFFFFFFFF0054FF"
	$"FF2A002A002A002AFF9F16FF002A54FF"
	$"FF002A002A002A00FF169FFF2A0054FF"
	$"FFFFFFFFFFFFFFFFFF9F16FFFFFFFFFF"
	$"0000000000000000FFFFFFFF00000000"
	};

resource 'ics4' (privateFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF0FFFF0000"
	$"FCCCCCCFFDDFFFF0"
	$"F0C0C0C0FDDFC0CF"
	$"FC0C0C0CFDDF0CCF"
	$"F0C0C0CFFFFFF0CF"
	$"FC0C0C0F0000FCCF"
	$"F0C0C0CF0FF0F0CF"
	$"FC0C0C0F0FF0FCCF"
	$"F0C0C0CFFFFFF0CF"
	$"FC0C0C0CFDDF0CCF"
	$"F0C0C0C0FDDFC0CF"
	$"FFFFFFFFFDDFFFFF"
	$"00000000FFFF0000"
	};

resource 'SICN' (privateFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 1E00 35F0 2AA8 3558"
	$"2AA8 3558 2AA8 3558 3FF8 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// owned folder (on server, able to set access privileges) icon (dark tab)

resource 'ICN#' (ownedFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 FFFFFFFE";
	
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (ownedFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 7E00 FFFE 8001 8001"
	$"8001 8001 8001 8001 8001 8001 8001 FFFF";
	
	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (ownedFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (ownedFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FFFFFFFFF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (ownedFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FFFFFFFFFFFF000000000000000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A002AFF"
	$"FF2A002A002A002A002A002A002A2AFF"
	$"FF002A002A002A002A002A002A002AFF"
	$"FF2A002A002A002A002A002A002A2AFF"
	$"FF002A002A002A002A002A002A002AFF"
	$"FF2A002A002A002A002A002A002A2AFF"
	$"FF002A002A002A002A002A002A002AFF"
	$"FF2A002A002A002A002A002A002A2AFF"
	$"FF002A002A002A002A002A002A002AFF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (ownedFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FFFFFF000000000"
	$"FFFFFFFFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// drop folder (write only access) icon (downward pointing arrow)

resource 'ICN#' (dropFolderIconResource, purgeable)
	{ {
	$"00000000 000001C0 000001C0 000001C0"
	$"000001C0 000007F0 07F003E0 080801C0"
	$"10040080 2002FE00 7FFF83FC 80008202"
	$"80008202 80008202 80008202 80008202"
	$"8001FF02 80010102 80017D02 80014502"
	$"80015502 80016D02 80012902 8001EF02"
	$"8000AA02 80009202 80008202 80008202"
	$"80008202 80008202 FFFF83FE 0000FE00";

	$"000003E0 000003E0 000003E0 000003E0"
	$"00001FFC 00000FF8 07F007F0 0FF803E0"
	$"1FFC01C0 3FFEFE80 7FFFFFFC FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE 0000FE00"
	} };

resource 'ics#' (dropFolderIconResource, purgeable)
	{ {
	$"0004 001F 3C0E 42E4 FF9E 8091 8091 81F9"
	$"8109 8169 8169 81F9 8091 8091 FF9F 00F0";

	$"0004 001F 3C0E 7EE4 FFFE FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 00F0"
	} };

resource 'icl8' (dropFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000FFFFFF000000000000"
	$"0000000000000000000000000000000000000000000000FFFFFF000000000000"
	$"0000000000000000000000000000000000000000000000FFFFFF000000000000"
	$"0000000000000000000000000000000000000000000000FFFFFF000000000000"
	$"000000000000000000000000000000000000000000FFFFFFFFFFFFFF00000000"
	$"0000000000FFFFFFFFFFFFFF00000000000000000000FFFFFFFFFF0000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000FFFFFF000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF00000000000000000000FF00000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF00FFFFFFFFFFFFFF000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFF169F169F16FFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF9F169F169FFF2A2A2A2A2A2A2AFF00"
	$"FF000000000000000000000000000000FF169F169F16FF00000000000000FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF169F169F16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FFFFFFFFFFFFFFFFFF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFF01010101010101FF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FF01FFFFFFFFFF01FF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFF01FF169F16FF01FF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FF01FF9FFF9FFF01FF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFF01FFFF01FFFF01FF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A00FF0101FF01FF0101FF002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AFFFFFFFF01FFFFFFFF2A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF16FF01FF16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F16FF169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF169F169F16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AFF169F169F16FF2A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00FF9F169F169FFF002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF169F169F16FFFFFFFFFFFFFFFFFF00"
	$"00000000000000000000000000000000FFFFFFFFFFFFFF000000000000000000"
	};

resource 'icl4' (dropFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000FFF000000"
	$"00000000000000000000000FFF000000"
	$"00000000000000000000000FFF000000"
	$"00000000000000000000000FFF000000"
	$"000000000000000000000FFFFFFF0000"
	$"00000FFFFFFF0000000000FFFFF00000"
	$"0000FCCCCCCCF0000000000FFF000000"
	$"000FCCCCCCCCCF0000000000F0000000"
	$"00FCCCCCCCCCCCF0FFFFFFF000000000"
	$"0FCCCCCCCCCCCCCFFDDDDDFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCFDDDDDFCCCCCCCF0"
	$"F000000000000000FDDDDDF0000000F0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDDDDDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0FFFFFFFFF0C0C0CF0"
	$"F0C0C0C0C0C0C0CF0000000FC0C0CCF0"
	$"FC0C0C0C0C0C0C0F0FFFFF0F0C0C0CF0"
	$"F0C0C0C0C0C0C0CF0FDDDF0FC0C0CCF0"
	$"FC0C0C0C0C0C0C0F0FDFDF0F0C0C0CF0"
	$"F0C0C0C0C0C0C0CF0FF0FF0FC0C0CCF0"
	$"FC0C0C0C0C0C0C0F00F0F00F0C0C0CF0"
	$"F0C0C0C0C0C0C0CFFFF0FFFFC0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDF0FDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDFDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDDDDDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FC0C0C0C0C0C0C0CFDDDDDFC0C0C0CF0"
	$"F0C0C0C0C0C0C0C0FDDDDDF0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFDDDDDFFFFFFFFF0"
	$"0000000000000000FFFFFFF000000000"
	};

resource 'ics8' (dropFolderIconResource, purgeable)
	{
	$"00000000000000000000000000FF0000"
	$"0000000000000000000000FFFFFFFFFF"
	$"0000FFFFFFFF000000000000FFFFFF00"
	$"00FF2A2A2A2AFF00FFFFFF0000FF0000"
	$"FF2A2A2A2A2A2AFFFF9F16FFFFFFFF00"
	$"FF002A002A002A00FF169FFF2A0054FF"
	$"FF2A002A002A002AFF9F16FF002A54FF"
	$"FF002A002A002AFFFFFFFFFFFF0054FF"
	$"FF2A002A002A00FF01010101FF2A54FF"
	$"FF002A002A002AFF01FFFF01FF0054FF"
	$"FF2A002A002A00FF01FFFF01FF2A54FF"
	$"FF002A002A002AFFFFFFFFFFFF0054FF"
	$"FF2A002A002A002AFF9F16FF002A54FF"
	$"FF002A002A002A00FF169FFF2A0054FF"
	$"FFFFFFFFFFFFFFFFFF9F16FFFFFFFFFF"
	$"0000000000000000FFFFFFFF00000000"
	};

resource 'ics4' (dropFolderIconResource, purgeable)
	{
	$"0000000000000F00"
	$"00000000000FFFFF"
	$"00FFFF000000FFF0"
	$"0FCCCCF0FFF00F00"
	$"FCCCCCCFFDDFFFF0"
	$"F0C0C0C0FDDFC0CF"
	$"FC0C0C0CFDDF0CCF"
	$"F0C0C0CFFFFFF0CF"
	$"FC0C0C0F0000FCCF"
	$"F0C0C0CF0FF0F0CF"
	$"FC0C0C0F0FF0FCCF"
	$"F0C0C0CFFFFFF0CF"
	$"FC0C0C0CFDDF0CCF"
	$"F0C0C0C0FDDFC0CF"
	$"FFFFFFFFFDDFFFFF"
	$"00000000FFFF0000"
	};

resource 'SICN' (dropFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 0070 1E20 35F0 2AA8 3558"
	$"2AA8 3558 2AA8 3558 3FF8 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// shared folder (by FileShare)

resource 'ICN#' (sharedFolderIconResource, purgeable)
	{ {
	$"07F00000 0FF80000 10040000 20020000"
	$"7FFFFFFC 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 80000002 80000002"
	$"80000002 80000002 82800002 82800002"
	$"F7DFFFFE 04400000 04400000 07C00000"
	$"05400000 B93FFFFD 02800000 BC7FFFFD";
	
	$"07F00000 0FF80000 1FFC0000 3FFE0000"
	$"7FFFFFFC FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE 07C00000 07C00000 07C00000"
	$"07C00000 BD7FFFFD BABFFFFD BC7FFFFD"
	} };

resource 'ics#' (sharedFolderIconResource, purgeable)
	{ {
	$"3C00 7E00 FFFE 8001 8001 8001 8001 8001"
	$"8001 8001 8801 8801 FFFF 0800 1400 F7FF";
	
	$"3C00 7E00 FFFE FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF 1C00 F7FF F7FF"
	} };

resource 'icl8' (sharedFolderIconResource, purgeable)
	{
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002AFE2BFC2A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A00FE2BFC002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFF00FF000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"0000000000FF2B2B2BFF00000000000000000000000000000000000000000000"
	$"0000000000FF2B2B2BFF00000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFF00000000000000000000000000000000000000000000"
	$"0000000000FD2BFE2BFF00000000000000000000000000000000000000000000"
	$"FCFCFCFDFD2B2BFE2B2BFCFCFCFBFBFBFAFAFAF9F9F9F8F8F8F7F7F7F6F6F6F6"
	$"2B2B2B2B2B2BFE00FE2B2B2B2B2B2B2B2B2B2BF6F6F6F6F6F6F6F5F5F5F5F5F5"
	$"FCFCFDFDFEFE000000FEFEFEFDFDFDFCFCFCFBFBFBFAFAFAF9F9F9F8F8F8F8F7"
	};

resource 'icl4' (sharedFolderIconResource, purgeable)
	{
	$"00000FFFFFFF00000000000000000000"
	$"0000FFFFFFFFF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0CFCEC0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0FCE0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFF0F000F0FFFFFFFFFFFFFFFFFFFF0"
	$"00000FCCCF0000000000000000000000"
	$"00000FCCCF0000000000000000000000"
	$"00000FFFFF0000000000000000000000"
	$"00000FCFCF0000000000000000000000"
	$"EEEEFCCFCCEEEEEEDDDDDDCCCCCCCCCC"
	$"CCCCCCF0FCCCCCCCCCCCCCCCCC000000"
	$"EEEEFF000FFFEEEEEEEEEDDDDDDCCCCC"
	};

resource 'ics8' (sharedFolderIconResource, purgeable)
	{
	$"0000FFFFFFFF00000000000000000000"
	$"00FFFFFFFFFFFF000000000000000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A00FF002A002A002A002A0054FF"
	$"FF2A002AFF2A002A002A002A002A54FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	$"00000000FF0000000000000000000000"
	$"000000FF00FF00000000000000000000"
	$"FFFFFFFF00FFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (sharedFolderIconResource, purgeable)
	{
	$"00FFFF0000000000"
	$"0FFFFFF000000000"
	$"FFFFFFFFFFFFFFF0"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0F0C0C0C0C0CF"
	$"FC0CFC0C0C0C0CCF"
	$"FFFFFFFFFFFFFFFF"
	$"0000F00000000000"
	$"000F0F0000000000"
	$"FFFF0FFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// mounted folder that has been shared and mounted by someone

resource 'ICN#' (mountedFolderIconResource, purgeable)
	{ {
	$"07F00000 0FF80000 10040000 20020000"
	$"7FFFFFFC 80000002 80000002 80FF8002"
	$"81004002 81024002 81027C02 81004202"
	$"81005202 81005202 8101C202 81008202"
	$"81048202 81038E02 81008402 80FF2402"
	$"80081C02 80080402 8007F802 82800002"
	$"F7DFFFFE 04400000 04400000 07C00000"
	$"05400000 B93FFFFD 02800000 BC7FFFFD";

	$"07F00000 0FF80000 1FFC0000 3FFE0000"
	$"7FFFFFFC FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE 07C00000 07C00000 07C00000"
	$"07C00000 FFFFFFFF FEFFFFFF FC7FFFFF"
	} };

resource 'ics#' (mountedFolderIconResource, purgeable)
	{ {
	$"3C00 7E00 FFFC 8002 8782 8482 85E2 8522"
	$"8762 8142 89C2 8802 FFFE 0800 1400 F7FF";

	$"3C00 7E00 FFFC FFFE FFFE FFFE FFFE FFFE"
	$"FFFE FFFE FFFE FFFE FFFE 0800 1400 F7FF"
	} };

resource 'icl8' (mountedFolderIconResource, purgeable)
	{
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FFFFFFFFFFFFFFFFFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A00FFFFFFFFFFFFFFFFFF002A002A002A002A002A002A54FF00"
	$"FF2A002A002A00FF333333333333333333FF002A002A002A002A002A0054FF00"
	$"FF002A002A002AFF333333333333FF3333FF2A002A002A002A002A002A54FF00"
	$"FF2A002A002A00FF333333333333FF3333FFFFFFFFFF002A002A002A0054FF00"
	$"FF002A002A002AFF333333333333333333FF08080808FF002A002A002A54FF00"
	$"FF2A002A002A00FF333333333333333333FF08FF0808FF2A002A002A0054FF00"
	$"FF002A002A002AFF333333333333333333FF08FF0808FF002A002A002A54FF00"
	$"FF2A002A002A00FF33333333333333FFFFFF08080808FF2A002A002A0054FF00"
	$"FF002A002A002AFF3333333333333333FF0808080808FF002A002A002A54FF00"
	$"FF2A002A002A00FF3333333333FF3333FF0808080808FF2A002A002A0054FF00"
	$"FF002A002A002AFF333333333333FFFFFF080808FFFFFF002A002A002A54FF00"
	$"FF2A002A002A00FF3333333333333333FF08080808FF002A002A002A0054FF00"
	$"FF002A002A002A00FFFFFFFFFFFFFFFF0808FF0808FF2A002A002A002A54FF00"
	$"FF2A002A002A002A002A002AFF080808080808FFFFFF002A002A002A0054FF00"
	$"FF002A002A002A002A002A00FF0808080808080808FF2A002A002A002A54FF00"
	$"FF2A002A002AFE2BFC2A002A00FFFFFFFFFFFFFFFF2A002A002A002A0054FF00"
	$"FF002A002A00FE2BFC002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFF00FF000000FF00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"0000000000FF2B2B2BFF00000000000000000000000000000000000000000000"
	$"0000000000FF2B2B2BFF00000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFF00000000000000000000000000000000000000000000"
	$"0000000000FD2BFE2BFF00000000000000000000000000000000000000000000"
	$"FCFCFCFDFD2B2BFE2B2BFCFCFCFBFBFBFAFAFAF9F9F9F8F8F8F7F7F7F6F6F6F6"
	$"2B2B2B2B2B2BFE00FE2B2B2B2B2B2B2B2B2B2BF6F6F6F6F6F6F6F5F5F5F5F5F5"
	$"FCFCFDFDFEFE000000FEFEFEFDFDFDFCFCFCFBFBFBFAFAFAF9F9F9F8F8F8F8F7"
	};

resource 'icl4' (mountedFolderIconResource, purgeable)
	{
	$"00000FFFFFFF00000000000000000000"
	$"0000FFFFFFFFF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0FFFFFFFFF0C0C0C0C0C0CCF0"
	$"FC0C0C0FBCBCBCBCBF0C0C0C0C0C0CF0"
	$"F0C0C0CFCBCBCBFBCFC0C0C0C0C0CCF0"
	$"FC0C0C0FBCBCBCFCBFFFFF0C0C0C0CF0"
	$"F0C0C0CFCBCBCBCBCF0202F0C0C0CCF0"
	$"FC0C0C0FBCBCBCBCBF2F20FC0C0C0CF0"
	$"F0C0C0CFCBCBCBCBCF0F02F0C0C0CCF0"
	$"FC0C0C0FBCBCBCBFFF2020FC0C0C0CF0"
	$"F0C0C0CFCBCBCBCBF20202F0C0C0CCF0"
	$"FC0C0C0FBCBCBFBCF02020FC0C0C0CF0"
	$"F0C0C0CFCBCBCBFFF202FFF0C0C0CCF0"
	$"FC0C0C0FBCBCBCBCF0202F0C0C0C0CF0"
	$"F0C0C0C0FFFFFFFF02F20FC0C0C0CCF0"
	$"FC0C0C0C0C0CF020202FFF0C0C0C0CF0"
	$"F0C0C0C0C0C0F20202020FC0C0C0CCF0"
	$"FC0C0CFCEC0C0FFFFFFFFC0C0C0C0CF0"
	$"F0C0C0FCE0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFF0F000F0FFFFFFFFFFFFFFFFFFFF0"
	$"00000FCCCF0000000000000000000000"
	$"00000FCCCF0000000000000000000000"
	$"00000FFFFF0000000000000000000000"
	$"00000ECFCF0000000000000000000000"
	$"EEEEECCFCCEEEEEEDDDDDDCCCCCCCCCC"
	$"CCCCCCF0FCCCCCCCCCCCCCCCCC000000"
	$"EEEEFF000FFFEEEEEEEEEDDDDDDCCCCC"
	};

resource 'ics8' (mountedFolderIconResource, purgeable)
	{
	$"0000FFFFFFFF00000000000000000000"
	$"00FFFFFFFFFFFF000000000000000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A002A002A002A002A002A002AFF00"
	$"FF002A002AFFFFFFFF002A002A00FF00"
	$"FF2A002A00FF3333FF2A002A002AFF00"
	$"FF002A002AFF33FFFFFFFF002A00FF00"
	$"FF2A002A00FF33FF0808FF2A002AFF00"
	$"FF002A002AFFFFFF08FFFF002A00FF00"
	$"FF2A002A002A00FF08FF002A002AFF00"
	$"FF002A00FF002AFFFFFF2A002A00FF00"
	$"FF2A002AFF2A002A002A002A002AFF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"00000000FF0000000000000000000000"
	$"000000FF00FF00000000000000000000"
	$"FFFFFFFF00FFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (mountedFolderIconResource, purgeable)
	{
	$"00FFFF0000000000"
	$"0FFFFFF000000000"
	$"FFFFFFFFFFFFFF00"
	$"FC0C0C0C0C0C0CF0"
	$"F0C0CFFFF0C0C0F0"
	$"FC0C0FBBFC0C0CF0"
	$"F0C0CFBFFFF0C0F0"
	$"FC0C0FBF22FC0CF0"
	$"F0C0CFFF2FF0C0F0"
	$"FC0C0C0F2F0C0CF0"
	$"F0C0F0CFFFC0C0F0"
	$"FC0CFC0C0C0C0CF0"
	$"FFFFFFFFFFFFFFF0"
	$"0000F00000000000"
	$"000F0F0000000000"
	$"FFFF0FFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// floppy disk

resource 'ICN#' (floppyIconResource, purgeable)
	{ {
	$"7FFFFFF0 81000108 81007104 81008902"
	$"81008901 81008901 81008901 81008901"
	$"81007101 81000101 80FFFE01 80000001"
	$"80000001 80000001 87FFFFE1 88000011"
	$"88000011 88000011 88000011 88000011"
	$"88000011 88000011 88000011 88000011"
	$"88000011 88000011 88000011 88000011"
	$"88000011 88000011 88000011 7FFFFFFE";

	$"7FFFFFF0 FFFFFFF8 FFFFFFFC FFFFFFFE"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF 7FFFFFFE"
	} };

resource 'ics#' (floppyIconResource, purgeable)
	{ {
	$"7FFC 900A 9069 9009 8FF1 8001 9FF9 A005"
	$"A005 A005 A005 A005 A005 A005 A005 FFFF";

	$"FFFC FFFE FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (floppyIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F52B2BFAFF000000"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9F9FDFDFDF9F9F9F9F52B2B2BFAFF0000"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9FD2B2B2BF5F9F9F9F52B2B2B2BFAFF00"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9FD2B2B2BF5F9F9F9F52B2B2B2B2BFAFF"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9FD2B2B2BF5F9F9F9F52B2B2B2B2B2BFF"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9FD2B2B2BF5F9F9F9F52B2B2B2B2B2BFF"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9FD2B2B2BF5F9F9F9F52B2B2B2B2B2BFF"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9FD2B2B2BF5F9F9F9F52B2B2B2B2B2BFF"
	$"FF2B2B2BF82B2BF9F9F9F9F9F9F9F9F9F9F5F5F52BF9F9F9F52B2B2B2B2B2BFF"
	$"FF2B2B2BF82B2B2BF9F9F9F9F9F9F9F9F9F9F9F9F9F9F92BF52B2B2B2B2B2BFF"
	$"FF2B2B2B2BF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F52B2B2B2B2B2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2B2B2B2BF8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F82B2B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2BF9F9F82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2BF9F9F82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"FF2B2B2BF82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF52B2B2BFF"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (floppyIconResource, purgeable)
	{
	$"0FFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FCCCDCCDDDDDDDDDDDDDDDDD0CCDF000"
	$"FCCCDCCDDDDDDDDDDEEEDDDD0CCCDF00"
	$"FCCCDCCDDDDDDDDDECCC0DDD0CCCCDF0"
	$"FCCCDCCDDDDDDDDDECCC0DDD0CCCCCDF"
	$"FCCCDCCDDDDDDDDDECCC0DDD0CCCCCCF"
	$"FCCCDCCDDDDDDDDDECCC0DDD0CCCCCCF"
	$"FCCCDCCDDDDDDDDDECCC0DDD0CCCCCCF"
	$"FCCCDCCDDDDDDDDDECCC0DDD0CCCCCCF"
	$"FCCCDCCDDDDDDDDDD000CDDD0CCCCCCF"
	$"FCCCDCCCDDDDDDDDDDDDDDDC0CCCCCCF"
	$"FCCCC00000000000000000000CCCCCCF"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FCCCCDDDDDDDDDDDDDDDDDDDDDDCCCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCDDDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCDDDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"FCCCDCCCCCCCCCCCCCCCCCCCCCC0CCCF"
	$"0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (floppyIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2B2BFBF9F9F9F9F9F9F9F9F52BFF00"
	$"FF2B2BFBF9F9F9F9F9FFFFF9F52B2BFF"
	$"FF2B2BFBF9F9F9F9F9F9F9F9F52B2BFF"
	$"FF2B2B2BF5F5F5F5F5F5F5F52B2B2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2B2BF8F8F8F8F8F8F8F8F8F82B2BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FF2BF82B2B2B2B2B2B2B2B2B2BF52BFF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (floppyIconResource, purgeable)
	{
	$"0FFFFFFFFFFFFF00"
	$"FCCEDDDDDDDD0CF0"
	$"FCCEDDDDDFFD0CCF"
	$"FCCEDDDDDDDD0CCF"
	$"FCCC00000000CCCF"
	$"FCCCCCCCCCCCCCCF"
	$"FCCDDDDDDDDDDCCF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FCDCCCCCCCCCC0CF"
	$"FFFFFFFFFFFFFFFF"
	};

resource 'SICN' (floppyIconResource, purgeable)
	{ {
	$"0000 0000 3FF0 48A8 48A4 4824 47C4 4004"
	$"4004 47C4 4824 4824 4824 3FFC 0000 0000";

	$"0000 0000 3FF0 7FF8 7FFC 7FFC 7FFC 7FFC"
	$"7FFC 7FFC 7FFC 7FFC 7FFC 3FFC 0000 0000"
	} };

//__________________________________________________________________________________________________
// hard disk

resource 'SICN' (genericHardDiskIconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 0000 7FFE 8001 8001"
	$"A001 8001 7FFE 0000 0000 0000 0000 0000";

	$"0000 0000 0000 0000 0000 7FFE FFFF FFFF"
	$"FFFF FFFF 7FFE 0000 0000 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// trash

resource 'ICN#' (trashIconResource, purgeable)
	{ {
	$"0007E000 00081000 03FFFFC0 04000020"
	$"07FFFFE0 02000040 02000040 02444440"
	$"02222240 02222240 02222240 02222240"
	$"02222240 02222240 02222240 02222240"
	$"02222240 02222240 02222240 02222240"
	$"02222240 02222240 02222240 02222240"
	$"02222240 02222240 02222240 02222240"
	$"02444440 02000040 02000040 01FFFF80";

	$"0007E000 000FF000 03FFFFC0 07FFFFE0"
	$"07FFFFE0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 01FFFF80"
	} };

resource 'ics#' (trashIconResource, purgeable)
	{ {
	$"0380 0440 7FFC 4004 7FFC 2008 2548 2548"
	$"2548 2548 2548 2548 2548 2548 2008 1FF0";

	$"0380 0440 7FFC 7FFC 7FFC 3FF8 3FF8 3FF8"
	$"3FF8 3FF8 3FF8 3FF8 3FF8 3FF8 3FF8 1FF0"
	} };

resource 'icl8' (trashIconResource, purgeable)
	{
	$"00000000000000000000000000FFFFFFFFFFFF00000000000000000000000000"
	$"000000000000000000000000FFFAF6F6F8FAFCFF000000000000000000000000"
	$"000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000"
	$"0000000000FFFCFAFAF8F8F600F6F6F8F8F9F9FAFAFAFBFBFCFCFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"000000000000FFFAF8F7F6F6F6F7F7F7F8F8F8F8FAFAFBFBFBFF000000000000"
	$"000000000000FFFAF8F7000000F6F6F6F6F8F8F8F8F8FAFBFBFF000000000000"
	$"000000000000FFFAF8F6000000FAF6F6F6F8FBF8F8FAFAFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFAFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F8F600FAF8F8F6F6FBFAFAF8FAFDFBFBFF000000000000"
	$"000000000000FFFAF8F6000000FAF6F6F6F8FBF8F8FAFBFAFBFF000000000000"
	$"000000000000FFFAF8F7000000F6F6F6F6F6F8F8F8FAFAFBFBFF000000000000"
	$"000000000000FFFAF8F6000000F6F6F7F7F8F8FAFAFAFBFBFBFF000000000000"
	$"00000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000"
	};

resource 'icl4' (trashIconResource, purgeable)
	{
	$"0000000000000FFFFFF0000000000000"
	$"000000000000FDCCCDEF000000000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	$"00000FEDDCC000CCDDDDDEEEEEF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"000000FDCCCCCCCCCCDDDDEEEF000000"
	$"000000FD0000000CCCCCCCDEEF000000"
	$"000000FDC0000D00CCECCDFDEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDCC00DCC0CEDDCDFEEF000000"
	$"000000FDC0000D00CCECCDFDEF000000"
	$"000000FDC0000000CCCCCCDDEF000000"
	$"000000FD0000000CCCCCCDDEEF000000"
	$"0000000FFFFFFFFFFFFFFFFFF0000000"
	};

resource 'ics8' (trashIconResource, purgeable)
	{
	$"000000000000FFFFFF00000000000000"
	$"0000000000FFF5F8FBFF000000000000"
	$"00FAFFFFFFFFFFFFFFFFFFFFFFFA0000"
	$"00FFFAF8F5F5F6F8F8FAFBFDFDFF0000"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"0000FFFAF6FAFAFCFCFCFDFDFF000000"
	$"0000FFF80000F6F6F8F8FAFBFF000000"
	$"0000FFF800F8F6FBF8FDFAFDFF000000"
	$"0000FFF800F8F6FBF8FDFAFDFF000000"
	$"0000FFF800F8F6FBF8FDFAFDFF000000"
	$"0000FFF800F8F6FBF8FDFAFDFF000000"
	$"0000FFF800F8F6FBF8FDFAFDFF000000"
	$"0000FFF800F8F6FBF8FDFAFDFF000000"
	$"0000FFF80000F6F6F8F8FAFBFF000000"
	$"0000FFFA2BF6F8F8FBFBFBFDFF000000"
	$"000000FFFFFFFFFFFFFFFFFF00000000"
	};

resource 'ics4' (trashIconResource, purgeable)
	{
	$"000000FFF0000000"
	$"00000FCCDF000000"
	$"0DFFFFFFFFFFFD00"
	$"0FDD000CDDDEEF00"
	$"0FFFFFFFFFFFFF00"
	$"00FDCCDDDEEFF000"
	$"00FD000CCDDEF000"
	$"00FD0C0DCEDFF000"
	$"00FD0C0DCEDFF000"
	$"00FD0C0DCEDFF000"
	$"00FD0C0DCEDFF000"
	$"00FD0C0DCEDFF000"
	$"00FD0C0DCEDFF000"
	$"00FD000CCDDEF000"
	$"00FDCCCDDEEEF000"
	$"000FFFFFFFFF0000"
	};

resource 'SICN' (trashIconResource, purgeable)
	{ {
	$"0000 03C0 1FF8 3FFC 0810 0A50 0A50 0A50"
	$"0A50 0A50 0A50 0810 0FF0 0000 0000 0000";

	$"0000 03C0 1FF8 3FFC 0FF0 0FF0 0FF0 0FF0"
	$"0FF0 0FF0 0FF0 0FF0 0FF0 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// full trash

resource 'ICN#' (fullTrashIconResource, purgeable)
	{ {
	$"0007E000 00081000 03FFFFC0 04000020"
	$"07FFFFE0 02000040 02000040 02100440"
	$"04222220 04222220 08441110 08441110"
	$"10880888 10880888 10880888 10880888"
	$"10880888 10880888 10880888 10880888"
	$"10880888 10880888 10880888 08441110"
	$"08441110 04222220 04222220 04222220"
	$"02100040 02000040 02000040 01FFFF80";

	$"0007E000 000FF000 03FFFFC0 07FFFFE0"
	$"07FFFFE0 03FFFFC0 03FFFFC0 03FFFFC0"
	$"07FFFFE0 07FFFFE0 0FFFFFF0 0FFFFFF0"
	$"1FFFFFF8 1FFFFFF8 1FFFFFF8 1FFFFFF8"
	$"1FFFFFF8 1FFFFFF8 1FFFFFF8 1FFFFFF8"
	$"1FFFFFF8 1FFFFFF8 1FFFFFF8 0FFFFFF0"
	$"0FFFFFF0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"03FFFFC0 03FFFFC0 03FFFFC0 01FFFF80"
	} };

resource 'ics#' (fullTrashIconResource, purgeable)
	{ {
	$"0380 0440 3FF8 2008 3FF8 1010 2448 2928"
	$"4924 4924 4924 4924 2928 2448 1010 0FE0";

	$"0380 0440 3FF8 3FF8 3FF8 1FF0 3FF8 3FF8"
	$"7FFC 7FFC 7FFC 7FFC 3FF8 3FF8 1FF0 0FE0"
	} };

resource 'icl8' (fullTrashIconResource, purgeable)
	{
	$"00000000000000000000000000FFFFFFFFFFFF00000000000000000000000000"
	$"000000000000000000000000FFFAF6F6F8F9FCFF000000000000000000000000"
	$"000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000"
	$"0000000000FFFCFAFAF8F8F600F6F6F8F8F9F9FAFAFAFBFBFCFCFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"000000000000FFFAF8F6F6F6F6F7F7F7F8F8F8F8FAFAFBFCFCFF000000000000"
	$"000000000000FFFAF8F5F5F5F5F6F6F6F6F8F8F8F8FAFBFBFCFF000000000000"
	$"000000000000FFFAF8F6F6F5F5F6FAF6F6F6FBF8F8FAFBFBFAFF000000000000"
	$"0000000000FFFAF8F6F8F6F5F5FAF7F7F6FBFAFAF8FAFDFAFBFAFF0000000000"
	$"0000000000FFFAF6F6F8F60000FAF82BF6FBFAFAF8FAFDFBFCFAFF0000000000"
	$"00000000FFFAF8F6F8F6F600FAF72B2BF6F6FBFAFAF8FAFDFBFCFAFF00000000"
	$"00000000FFFAF6F6F8F60000FAF7F600F5F6FBFAFAF8FAFDFBFCFAFF00000000"
	$"000000FFFAF8F6F8F60000F9F7F60000F5F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F7F6000000F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F7F6000000F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F7F6000000F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F7F6000000F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F7F6000000F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F8F6000000F5F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F8F60000F5F6F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F8F700F5F6F6F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F60000F9F8F7F5F6F6F6F6FBFAFAF8FAFDFBFCFAFF000000"
	$"000000FFFAF8F6F8F7F6F6F9F8F7F6F6F6F6F6FBFAFAF8FAFDFBFCFAFF000000"
	$"00000000FFFAF8F6F8F7F6F6F9F8F7F6F6F6FBFAFAF8F8FDFBFCFAFF00000000"
	$"00000000FFFAF8F6F8F7F6F6F9F8F7F6F6F6FBFAFAF8FAFDFBFCFAFF00000000"
	$"0000000000FFFAF6F8F7F6F6F6F9F8F7F6FBFAFAF8F8FAFDFCFAFF0000000000"
	$"0000000000FFFAF8F6F8F6F6F6F9F8F72BFBFAFAF8FAFDFBFCFAFF0000000000"
	$"0000000000FFFAF8F6F82BF62BF9F8F72BFBFAFAF8FAFDFCFCFAFF0000000000"
	$"000000000000FFFAF7F6F72B2BF9F82BF7FBFAF8F9FAFBFCFAFF000000000000"
	$"000000000000FFFAF80000F62B2B2BF7F8F8F8F9FAFAFBFCFAFF000000000000"
	$"000000000000FFFAF8F8F8F8F8F8F8F8F9F9FAFAFAFBFCFCFBFF000000000000"
	$"00000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000"
	};

resource 'icl4' (fullTrashIconResource, purgeable)
	{
	$"0000000000000FFFFFF0000000000000"
	$"000000000000FDCCCDEF000000000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	$"00000FEDDCC000CCDDDDDEEEEEF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"000000FEDDCCCCCCCCDDDDEEEF000000"
	$"000000FED00000CCCCCCCDEEEF000000"
	$"000000FEC0C000DCCCECCDDEEF000000"
	$"00000FEDCDC00DCCCEDDCDFDEEF00000"
	$"00000FEDCDC00DCCCEDDCCFEEEF00000"
	$"0000FEDCDCC0DCC0CCEDDCDFEEEF0000"
	$"0000FEDCDC00DCC0CCEDDCDFEEEF0000"
	$"000FEDCDCC0DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDC00DCC000CCEDDCDFEEEF000"
	$"000FEDCDCC0DCC00CCCEDDCDFEEEF000"
	$"0000FEDCDC0CDCCCCCEDDCDFEEEF0000"
	$"0000FEDCDCCCDCCCCCEDDCDFEEEF0000"
	$"00000FEDCDCCCDCCCEDDCCDFEEF00000"
	$"00000FEDCDCCCDCCCEDDCDFEEEF00000"
	$"00000FEDCDCCCDCCCEDDCDFEEEF00000"
	$"000000FEDCCCCCCCCCCDDDDEEF000000"
	$"000000FEDDDCCCCCDDDDDDEEEF000000"
	$"000000FEEDDDDDDDDDEEEEEEFF000000"
	$"0000000FFFFFFFFFFFFFFFFFF0000000"
	};

resource 'ics8' (fullTrashIconResource, purgeable)
	{
	$"000000000000FFFFFF00000000000000"
	$"0000000000FFF6F8FAFF000000000000"
	$"000000FFFFFFFFFFFFFFFFFF00000000"
	$"0000FFFAF80000F8FAFAFBFDFF000000"
	$"0000FFFFFFFFFFFFFFFFFFFFFF000000"
	$"000000FFF800F8F8FAFAFBFF00000000"
	$"0000FFF800FA00F6F8FDFAFBFF000000"
	$"0000FF00FA0000FBF8F8FDFAFF000000"
	$"00FFF800FA0000FBF8F8FDFAFBFF0000"
	$"00FFF800FA00F6FBF8F8FDFAFBFF0000"
	$"00FFF800FA00F6FBF8F8FDFAFBFF0000"
	$"00FFF800FA00F6FBF8F8FDFAFBFF0000"
	$"0000FFF6FAF6F6FBF8F8FDFAFF000000"
	$"0000FFF8F6FAF8F8F8FDFAFBFF000000"
	$"000000FFF8F6F8F8FAFAFBFF00000000"
	$"00000000FFFFFFFFFFFFFF0000000000"
	};

resource 'ics4' (fullTrashIconResource, purgeable)
	{
	$"000000FFF0000000"
	$"00000FCCDF000000"
	$"000FFFFFFFFF0000"
	$"00FED00DDDEEF000"
	$"00FFFFFFFFFFF000"
	$"000FDCDDEEEF0000"
	$"00FD0D0CDEDEF000"
	$"00F0D00ECDEDF000"
	$"0FD0D00ECDEDEF00"
	$"0FD0D0CECDEDEF00"
	$"0FD0D0CECDEDEF00"
	$"0FD0D0CECDEDEF00"
	$"00F0DCCECDEDF000"
	$"00FD0DCCDEDEF000"
	$"000FDCCDEEEF0000"
	$"0000FFFFFFF00000"
	};

//__________________________________________________________________________________________________
// desktop

resource 'ics#' (desktopIconResource, purgeable)
	{ {
	$"0000 0000 7FFE 8001 FFFF AAAB D555 AAAB"
	$"D555 AAAB D555 AAAB D555 AAAB D555 7FFE";

	$"0000 0000 7FFE FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 7FFE"
	} };

resource 'SICN' (desktopIconResource, purgeable)
	{ {
	$"0000 3FFC 4002 4002 7FFE 6AAA 5556 6AAA"
	$"5556 6AAA 5556 6AAA 3FFC 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// popup menu symbol

resource 'SICN' (popupMenuSymbolResource, purgeable)
	{ {
	$"0000 0000 0000 0000 0000 0000 3FF8 1FF0"
	$"0FE0 07C0 0380 0100 0000 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// standalone (with border) popup menu symbol

resource 'SICN' (standalonePopupMenuSymbolResource, purgeable)
	{ {
	$"0000 0000 FFF8 800C 800C BFEC 9FCC 8F8C"
	$"870C 820C 800C FFFC 7FFC 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// System Folder

resource 'ICN#' (systemFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 803FF002"
	$"80400802 805FE802 80502802 80552802"
	$"80502802 80542802 80502802 805FE802"
	$"80400802 80400802 8041E802 80400802"
	$"803FF002 803FF002 80000002 FFFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (systemFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 87E1"
	$"8421 85A1 8521 8421 8421 83C1 8001 FFFF";

	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (systemFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002AFFFFFFFFFFFFFFFFFFFF002A002A002A002A0054FF00"
	$"FF002A002A002A002AFF2B2B2B2B2B2B2B2B2B2BFF002A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2BFCFCFCFCFCFCFCFC2BFF2A002A002A002A0054FF00"
	$"FF002A002A002A002AFF2BFC2A2A2A2A2A2A002BFF002A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2BFC2A2A2A2A2A2A002BFF2A002A002A002A0054FF00"
	$"FF002A002A002A002AFF2BFC2A2A2A2A2A2A002BFF002A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2BFC2A2A2A2A2A2A002BFF2A002A002A002A0054FF00"
	$"FF002A002A002A002AFF2BFC2A2A2A2A2A2A002BFF002A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2BFA000000000000002BFF2A002A002A002A0054FF00"
	$"FF002A002A002A002AFF2B2B2B2B2B2B2B2B2B2BFF002A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2BE32B2B2B2B2B2B2B2BFF2A002A002A002A0054FF00"
	$"FF002A002A002A002AFF2BD82B2B2BFFFFFFFF2BFF002A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2B2B2B2B2B2B2B2B2B2BFF2A002A002A002A0054FF00"
	$"FF002A002A002A002A00FFFDFDFDFDFDFDFDFDFF2A002A002A002A002A54FF00"
	$"FF2A002A002A002A002AFFFFFFFFFFFFFFFFFFFF002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (systemFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C00FFFFFFFFFF0C0C0C0C0CF0"
	$"F0C0C0C0CFCCCCCCCCCCF0C0C0C0CCF0"
	$"FC0C0C0C0FCEEEEEEEECFC0C0C0C0CF0"
	$"F0C0C0C0CFCED0D0D0CCF0C0C0C0CCF0"
	$"FC0C0C0C0FCE0D0D0D0CFC0C0C0C0CF0"
	$"F0C0C0C0CFCED0D0D0CCF0C0C0C0CCF0"
	$"FC0C0C0C0FCE0D0D0D0CFC0C0C0C0CF0"
	$"F0C0C0C0CFCED0D0D0CCF0C0C0C0CCF0"
	$"FC0C0C0C0FCD0C0C0C0CFC0C0C0C0CF0"
	$"F0C0C0C0CFCCCCCCCCCCF0C0C0C0CCF0"
	$"FC0C0C0C0FC8CCCCCCCCFC0C0C0C0CF0"
	$"F0C0C0C0CFC3CCCFFFFCF0C0C0C0CCF0"
	$"FC0C0C0C0FCCCCCCCCCCFC0C0C0C0CF0"
	$"F0C0C0C0CCFFFFFFFFFF00C0C0C0CCF0"
	$"FC0C0C0C00FFFFFFFFFF0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (systemFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A00FFFFFFFFFFFF2A002A54FF"
	$"FF002A002AFF2B2B2B2BFF002A0054FF"
	$"FF2A002A00FF2BFCFCF5FF2A002A54FF"
	$"FF002A002AFF2BFC2AF5FF002A0054FF"
	$"FF2A002A00FF2BF5F5F5FF2A002A54FF"
	$"FF002A002AFF2B2B2B2BFF002A0054FF"
	$"FF2A002A002AFFFFFFFF002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (systemFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0FFFFFFC0CCF"
	$"F0C0CFCCCCF0C0CF"
	$"FC0C0FCEE0FC0CCF"
	$"F0C0CFCEC0F0C0CF"
	$"FC0C0FC000FC0CCF"
	$"F0C0CFCCCCF0C0CF"
	$"FC0C00FFFF0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Apple Menu Folder

resource 'ICN#' (appleMenuFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80006002"
	$"8000C002 80008002 800E7002 801FF802"
	$"803FF002 803FE002 803FE002 803FF002"
	$"803FF802 801FF802 801FF802 800FF002"
	$"80066002 80000002 80000002 FFFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (appleMenuFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3E00 4100 FFFF 8001 80C1"
	$"8281 87E1 87C1 87E1 87E1 8341 8001 FFFF";

	$"0000 0000 0000 3E00 7F00 FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (appleMenuFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A00E3E32A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A00E3E32A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AE32A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A00E3E392002A92E3E32A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A0092E3E3E3E3E3E3E3E3922A002A002A002A0054FF00"
	$"FF002A002A002A002A00080505050505050508002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A0805050505050505082A002A002A002A002A0054FF00"
	$"FF002A002A002A002A00161616161616161616082A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A16161616161616161616162A002A002A002A0054FF00"
	$"FF002A002A002A002A0016D8D8D8D8D8D8D8D8D8D8002A002A002A002A54FF00"
	$"FF2A002A002A002A002A2A696969696969696969692A002A002A002A0054FF00"
	$"FF002A002A002A002A002A5C69696969696969695C002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002AECECECECECECECEC002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002AECEC2A2AECEC002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (appleMenuFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C088C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C088C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C8C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C088C0CC88C0C0C0C0CCF0"
	$"FC0C0C0C0C0C88888888CC0C0C0C0CF0"
	$"F0C0C0C0C0C11111111CC0C0C0C0CCF0"
	$"FC0C0C0C0CC11111111C0C0C0C0C0CF0"
	$"F0C0C0C0C0222222222CC0C0C0C0CCF0"
	$"FC0C0C0C0C22222222222C0C0C0C0CF0"
	$"F0C0C0C0C0233333333330C0C0C0CCF0"
	$"FC0C0C0C0CC3535353535C0C0C0C0CF0"
	$"F0C0C0C0C0CD35353535D0C0C0C0CCF0"
	$"FC0C0C0C0C0C666666660C0C0C0C0CF0"
	$"F0C0C0C0C0C0C660C660C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (appleMenuFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFFFF000000000000000000"
	$"00FF2A2A2A2A2AFF0000000000000000"
	$"FF2A2A2A2A2A2A2AFFFFFFFFFFFFFFFF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002AE3E3002A002A54FF"
	$"FF002A002A00E300E3002A002A0054FF"
	$"FF2A002A000805050505082A002A54FF"
	$"FF002A002A16161616162A002A0054FF"
	$"FF2A002A00D8D8D8D8D8162A002A54FF"
	$"FF002A002A5C696969695C002A0054FF"
	$"FF2A002A002AECEC2AEC002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (appleMenuFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFFF000000000"
	$"0FCCCCCF00000000"
	$"FCCCCCCCFFFFFFFF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C880C0CCF"
	$"F0C0C08080C0C0CF"
	$"FC0C0C1111CC0CCF"
	$"F0C0C22222C0C0CF"
	$"FC0C0333333C0CCF"
	$"F0C0C3636360C0CF"
	$"FC0C0C6606CC0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Startup Folder

resource 'ICN#' (startupFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80000002"
	$"8007C002 80183002 80200802 80210802"
	$"80410402 80410402 80410402 80410402"
	$"80410402 80210802 80200802 80183002"
	$"8007C002 80000002 80000002 FFFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (startupFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 8381"
	$"8441 8921 8921 8921 8441 8381 8001 FFFF";

	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (startupFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A0054FFFFFFFFFF54002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A00FFFF2A002A002AFFFF002A002A002A002A0054FF00"
	$"FF002A002A002A002A00FF542A002A002A002A54FF002A002A002A002A54FF00"
	$"FF2A002A002A002A0054FF2A002A00FF002A002AFF54002A002A002A0054FF00"
	$"FF002A002A002A002AFF2A002A002AFF2A002A002AFF2A002A002A002A54FF00"
	$"FF2A002A002A002A00FF002A002A00FF002A002A00FF002A002A002A0054FF00"
	$"FF002A002A002A002AFF2A002A002AFF2A002A002AFF2A002A002A002A54FF00"
	$"FF2A002A002A002A00FF002A002A00FF002A002A00FF002A002A002A0054FF00"
	$"FF002A002A002A002AFF2A002A002AFF2A002A002AFF2A002A002A002A54FF00"
	$"FF2A002A002A002A0054FF2A002A00FF002A002AFF54002A002A002A0054FF00"
	$"FF002A002A002A002A00FF542A002A002A002A54FF002A002A002A002A54FF00"
	$"FF2A002A002A002A002A00FFFF2A002A002AFFFF002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A0054FFFFFFFFE054002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (startupFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0CFFFFFC0C0C0C0C0CCF0"
	$"FC0C0C0C0C0FFC0C0CFF0C0C0C0C0CF0"
	$"F0C0C0C0C0FCC0C0C0CCF0C0C0C0CCF0"
	$"FC0C0C0C0CFC0C0F0C0CFC0C0C0C0CF0"
	$"F0C0C0C0CFC0C0CFC0C0CFC0C0C0CCF0"
	$"FC0C0C0C0F0C0C0F0C0C0F0C0C0C0CF0"
	$"F0C0C0C0CFC0C0CFC0C0CFC0C0C0CCF0"
	$"FC0C0C0C0F0C0C0F0C0C0F0C0C0C0CF0"
	$"F0C0C0C0CFC0C0CFC0C0CFC0C0C0CCF0"
	$"FC0C0C0C0CFC0C0F0C0CFC0C0C0C0CF0"
	$"F0C0C0C0C0FCC0C0C0CCF0C0C0C0CCF0"
	$"FC0C0C0C0C0FFC0C0CFF0C0C0C0C0CF0"
	$"F0C0C0C0C0C0CFFFFFC0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (startupFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002AFDFDFD2A002A002A54FF"
	$"FF002A002AFD2A002AFD2A002A0054FF"
	$"FF2A002AFD2A00FD002AFD2A002A54FF"
	$"FF002A00FD002AFD2A00FD002A0054FF"
	$"FF2A002AFD2A00FD002AFD2A002A54FF"
	$"FF002A002AFD2A002AFD2A002A0054FF"
	$"FF2A002A002AFDFDFD2A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (startupFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0CFFFC0C0CCF"
	$"F0C0CFC0CFC0C0CF"
	$"FC0CFC0F0CFC0CCF"
	$"F0C0F0CFC0F0C0CF"
	$"FC0CFC0F0CFC0CCF"
	$"F0C0CFC0CFC0C0CF"
	$"FC0C0CFFFC0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Control Panels Folder

resource 'ICN#' (controlPanelFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 03F80000"
	$"04040000 08020000 10010000 3FFFFFFE"
	$"40000001 40000001 40000001 40000001"
	$"40000001 40000001 42222221 42222221"
	$"40000001 4003C001 47FE7FF1 48024009"
	$"47FE7FF1 4003C001 40000001 40000001"
	$"40000001 40000001 40000001 7FFFFFFF";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 03F80000"
	$"07FC0000 0FFE0000 1FFF0000 3FFFFFFE"
	$"7FFFFFFF 7FFFFFFF 7FFFFFFF 7FFFFFFF"
	$"7FFFFFFF 7FFFFFFF 7FFFFFFF 7FFFFFFF"
	$"7FFFFFFF 7FFFFFFF 7FFFFFFF 7FFFFFFF"
	$"7FFFFFFF 7FFFFFFF 7FFFFFFF 7FFFFFFF"
	$"7FFFFFFF 7FFFFFFF 7FFFFFFF 7FFFFFFF"
	} };

resource 'ics#' (controlPanelFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 8001"
	$"8001 9249 8001 8081 9EF9 8181 8001 FFFF";

	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (controlPanelFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"000000000000FFFFFFFFFFFFFF00000000000000000000000000000000000000"
	$"0000000000FF2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2A2A2AFF00000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF"
	$"00FF0000000000000000000000000000000000000000000000000000000000FF"
	$"00FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF"
	$"00FF00002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF"
	$"00FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF"
	$"00FF00002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF"
	$"00FF002A002AFB2A002AFB2A002AFB2A002AFB2A002AFB2A002AFB2A002A54FF"
	$"00FF00002A00FB002A00FB002A00FB002A00FB002A00FB002A00FB002A0054FF"
	$"00FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF"
	$"00FF00002A002A002A002A002A00000000FF2A002A002A002A002A002A0054FF"
	$"00FF002AFBFAFAFAFAFAFAFAFAFA00F6F6FFFFFAFAFAFAFAFAFAFAFA002A54FF"
	$"00FF0000FBF8F8F8F8F8F8F8F8F800F6F6FFFAF8F8F8F8F8F8F8F8F8000054FF"
	$"00FF002A0000000000000000000000F6F6FFF8000000000000000000002A54FF"
	$"00FF00002A002A002A002A002A00FFFFFFFFF8002A002A002A002A002A0054FF"
	$"00FF002A002A002A002A002A002AF8F8F8F8F82A002A002A002A002A002A54FF"
	$"00FF00002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF"
	$"00FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF"
	$"00FF00002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF"
	$"00FF0054545454545454545454545454545454545454545454545454545454FF"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'icl4' (controlPanelFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"000000FFFFFFF0000000000000000000"
	$"00000FCCCCCCCF000000000000000000"
	$"0000FCCCCCCCCCF00000000000000000"
	$"000FCCCCCCCCCCCF0000000000000000"
	$"00FCCCCCCCCCCCCCFFFFFFFFFFFFFFF0"
	$"0FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"0F00000000000000000000000000000F"
	$"0F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF"
	$"0F00C0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"0F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF"
	$"0F00C0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"0F0C0CEC0CEC0CEC0CEC0CEC0CEC0CCF"
	$"0F00C0E0C0E0C0E0C0E0C0E0C0E0C0CF"
	$"0F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF"
	$"0F00C0C0C0C0C0000FC0C0C0C0C0C0CF"
	$"0F0CEDDDDDDDDD0CCFFDDDDDDDDD0CCF"
	$"0F00ECCCCCCCCC0CCFDCCCCCCCCC00CF"
	$"0F0C00000000000CCFC0000000000CCF"
	$"0F00C0C0C0C0C0FFFFC0C0C0C0C0C0CF"
	$"0F0C0C0C0C0C0CCCCCCC0C0C0C0C0CCF"
	$"0F00C0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"0F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF"
	$"0F00C0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"0F0CCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics8' (controlPanelFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A00FB002AFB2A00FB002AFB2A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002AFF2A002A002A54FF"
	$"FF002AFBFBFBFB00FFFBFBFBFB0054FF"
	$"FF2A002A002A00FFFF2A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (controlPanelFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0E0CEC0E0CECCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0CFC0C0CCF"
	$"F0CEEEE0FEEEE0CF"
	$"FC0C0C0FFC0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Spool Folder

resource 'ICN#' (spoolFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80000002"
	$"801FC002 80106002 80107002 80101002"
	$"80101002 80101002 80101002 807FFC02"
	$"80400402 80401402 80400402 807FFC02"
	$"80000002 80000002 80000002 FFFFFFFE";
	
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (spoolFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 83C1"
	$"8261 8221 8221 87F1 8411 87F1 8001 FFFF";
	
	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (spoolFolderIconResource, purgeable) {
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002AFFFFFFFFFFFFFF2A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A00FFF5F5F5F5F5FFFF2A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002AFFF5F5F5F5F5FFFFFF2A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A00FFF5F5F5F5F5F5F5FF002A002A002A002A0054FF00"
	$"FF002A002A002A002A002AFFF5F5F5F5F5F5F5FF2A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A00FFF5F5F5F5F5F5F5FF002A002A002A002A0054FF00"
	$"FF002A002A002A002A002AFFF5F5F5F5F5F5F5FF2A002A002A002A002A54FF00"
	$"FF2A002A002A002A00FFFFFFFFFFFFFFFFFFFFFFFFFF002A002A002A0054FF00"
	$"FF002A002A002A002AFF2B2B2B2B2B2B2B2B2B2B2BFF2A002A002A002A54FF00"
	$"FF2A002A002A002A00FF2B2B2B2B2B2B2B2B2BD82BFF002A002A002A0054FF00"
	$"FF002A002A002A002AFF2B2B2B2B2B2B2B2B2B2B2BFF2A002A002A002A54FF00"
	$"FF2A002A002A002A00FFFFFFFFFFFFFFFFFFFFFFFFFF002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
};

resource 'icl4' (spoolFolderIconResource, purgeable) {
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0CFFFFFFFC0C0C0C0C0CCF0"
	$"FC0C0C0C0C0F00000FFC0C0C0C0C0CF0"
	$"F0C0C0C0C0CF00000FFFC0C0C0C0CCF0"
	$"FC0C0C0C0C0F0000000F0C0C0C0C0CF0"
	$"F0C0C0C0C0CF0000000FC0C0C0C0CCF0"
	$"FC0C0C0C0C0F0000000F0C0C0C0C0CF0"
	$"F0C0C0C0C0CF0000000FC0C0C0C0CCF0"
	$"FC0C0C0C0FFFFFFFFFFFFF0C0C0C0CF0"
	$"F0C0C0C0CFCCCCCCCCCCCFC0C0C0CCF0"
	$"FC0C0C0C0FCCCCCCCCC3CF0C0C0C0CF0"
	$"F0C0C0C0CFCCCCCCCCCCCFC0C0C0CCF0"
	$"FC0C0C0C0FFFFFFFFFFFFF0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
};

resource 'ics8' (spoolFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002AFFFFFFFF002A002A54FF"
	$"FF002A002A00FFF5F5FFFF002A0054FF"
	$"FF2A002A002AFFF5F5F5FF2A002A54FF"
	$"FF002A002A00FFF5F5F5FF002A0054FF"
	$"FF2A002A00FFFFFFFFFFFFFF002A54FF"
	$"FF002A002AFF2B2B2B2B2BFF2A0054FF"
	$"FF2A002A00FFFFFFFFFFFFFF002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (spoolFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0CFFFF0C0CCF"
	$"F0C0C0F00FF0C0CF"
	$"FC0C0CF000FC0CCF"
	$"F0C0C0F000F0C0CF"
	$"FC0C0FFFFFFF0CCF"
	$"F0C0CFCCCCCFC0CF"
	$"FC0C0FFFFFFF0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Preferences Folder

resource 'ICN#' (preferencesFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80E00002"
	$"81100002 82E80002 82E9F782 82E80002"
	$"81100002 80E00002 80000002 80E00002"
	$"81100002 82080002 8209EF82 82080002"
	$"81100002 80E00002 80000002 FFFFFFFE";
	
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (preferencesFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8C01 9E01"
	$"9EE1 8C01 8001 8C01 9201 92E1 8C01 FFFF";
	
	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (preferencesFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002AFFFFFF2A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002AFF000000FF2A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002AFF00FFFFFF00FF2A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A00FF00FFFFFF00FF002AFFFFFFFFFF2AFFFFFFFF002A002A54FF00"
	$"FF2A002A002AFF00FFFFFF00FF2A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002AFF000000FF2A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002AFFFFFF2A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002AFFFFFF2A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002AFF000000FF2A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002AFF0000000000FF2A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A00FF0000000000FF002AFFFFFFFF00FFFFFFFFFF002A002A54FF00"
	$"FF2A002A002AFF0000000000FF2A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002AFF000000FF2A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002AFFFFFF2A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (preferencesFolderIconResource, purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0FFF FFFF 0000 0000 0000 0000 0000"
	$"0000 FCCC CCCC F000 0000 0000 0000 0000"
	$"000F CCCC CCCC CF00 0000 0000 0000 0000"
	$"00FC CCCC CCCC CCF0 0000 0000 0000 0000"
	$"0FCC CCCC CCCC CCCF FFFF FFFF FFFF FF00"
	$"FCCC CCCC CCCC CCCC CCCC CCCC CCCC CCF0"
	$"F000 0000 0000 0000 0000 0000 0000 00F0"
	$"F0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FC0C 0C0C FFFC 0C0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0CF 000F C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FC0C 0CF0 FFF0 FC0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0F0 FFF0 F0CF FFFF CFFF F0C0 CCF0"
	$"FC0C 0CF0 FFF0 FC0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0CF 000F C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FC0C 0C0C FFFC 0C0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FC0C 0C0C FFFC 0C0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0CF 000F C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FC0C 0CF0 0000 FC0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0F0 0000 F0CF FFF0 FFFF F0C0 CCF0"
	$"FC0C 0CF0 0000 FC0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0CF 000F C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FC0C 0C0C FFFC 0C0C 0C0C 0C0C 0C0C 0CF0"
	$"F0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 CCF0"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFF0"
};


resource 'ics8' (preferencesFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A00FFFF2A002A002A002A0054FF"
	$"FF2A00FFFFFFFF2A002A002A002A54FF"
	$"FF002AFFFFFFFF00FFFFFF002A0054FF"
	$"FF2A002AFFFF002A002A002A002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002AFFFF002A002A002A002A54FF"
	$"FF002AFF0000FF002A002A002A0054FF"
	$"FF2A00FF0000FF2AFFFFFF2A002A54FF"
	$"FF002A00FFFF2A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (preferencesFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0FFC0C0C0C0CF"
	$"FC0FFFFC0C0C0CCF"
	$"F0CFFFF0FFF0C0CF"
	$"FC0CFF0C0C0C0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FC0CFF0C0C0C0CCF"
	$"F0CF00F0C0C0C0CF"
	$"FC0F00FCFFFC0CCF"
	$"F0C0FFC0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Extensions Folder

resource 'ICN#' (extensionsFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80000002"
	$"80FFF002 81000802 81000802 81800E02"
	$"81400902 80400102 81400902 81800E02"
	$"81000802 81000802 81F8F002 80104002"
	$"800F8002 80000002 80000002 FFFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (extensionsFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 9FF1"
	$"9011 9819 8809 9819 9011 9FF1 8001 FFFF";

	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (extensionsFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF00002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A00FFFFFFFFFFFFFFFFFFFFFFFF2A002A002A002A002A54FF00"
	$"FF00002A002A00FF000000000000000000000000FF2A002A002A002A0054FF00"
	$"FF002A002A002AFF00F6F6F6F6F6F6F6F6F6F6F6FF002A002A002A002A54FF00"
	$"FF00002A002A00FFFFF6F6F6F6F6F6F6F6F6F6F6FFFFFF2A002A002A0054FF00"
	$"FF002A002A002AFF2AFFF6F6F6F6F6F6F6F6F6F6FFF6F6FF2A002A002A54FF00"
	$"FF00002A002A002A00FFF6F6F6F6F6F6F6F6F6F6F6F6F6FF002A002A0054FF00"
	$"FF002A002A002AFF2AFFF6F6F6F6F6F6F6F6F6F6FFF6F6FF2A002A002A54FF00"
	$"FF00002A002A00FFFFF6F6F6F6F6F6F6F6F6F6F6FFFFFF2A002A002A0054FF00"
	$"FF002A002A002AFF00F6F6F6F6F6F6F6F6F6F6F6FF002A002A002A002A54FF00"
	$"FF00002A002A00FF00F6F6F6F6F6F6F6F6F6F6F6FF2A002A002A002A0054FF00"
	$"FF002A002A002AFFFFFFFFFFFFF6F6F6FFFFFFFF2A002A002A002A002A54FF00"
	$"FF00002A002A002A002A00FFF6F6F6F6F6FF002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A00FFFFFFFFFF002A002A002A002A002A002A54FF00"
	$"FF00002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF5454545454545454545454545454545454545454545454545454545454FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (extensionsFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0FFFFFFFFFFFFC0C0C0C0CCF0"
	$"F00C0C0F000000000000FC0C0C0C0CF0"
	$"F0C0C0CF0CCCCCCCCCCCF0C0C0C0CCF0"
	$"F00C0C0FFCCCCCCCCCCCFFFC0C0C0CF0"
	$"F0C0C0CFCFCCCCCCCCCCFCCFC0C0CCF0"
	$"F00C0C0C0FCCCCCCCCCCCCCF0C0C0CF0"
	$"F0C0C0CFCFCCCCCCCCCCFCCFC0C0CCF0"
	$"F00C0C0FFCCCCCCCCCCCFFFC0C0C0CF0"
	$"F0C0C0CF0CCCCCCCCCCCF0C0C0C0CCF0"
	$"F00C0C0F0CCCCCCCCCCCFC0C0C0C0CF0"
	$"F0C0C0CFFFFFFCCCFFFFC0C0C0C0CCF0"
	$"F00C0C0C0C0FCCCCCF0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0FFFFF0C0C0C0C0C0CCF0"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};


resource 'ics8' (extensionsFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A00FFFFFFFFFFFFFFFFFF002A54FF"
	$"FF002AFFF6F6F6F6F6F6F6FF2A0054FF"
	$"FF2A00FFFFF6F6F6F6F6F6FFFF2A54FF"
	$"FF002A00FFF6F6F6F6F6F6F6FF0054FF"
	$"FF2A00FFFFF6F6F6F6F6F6FFFF2A54FF"
	$"FF002AFFF6F6F6F6F6F6F6FF2A0054FF"
	$"FF2A00FFFFFFFFFFFFFFFFFF002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (extensionsFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000"
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0FFFFFFFFF0CCF"
	$"F0CFCCCCCCCFC0CF"
	$"FC0FFCCCCCCFFCCF"
	$"F0C0FCCCCCCCF0CF"
	$"FC0FFCCCCCCFFCCF"
	$"F0CFCCCCCCCFC0CF"
	$"FC0FFFFFFFFF0CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// Fonts Folder

resource 'ICN#' (fontsFolderIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"08080000 10040000 20020000 7FFFFFFC"
	$"80000002 80000002 80000002 80000802"
	$"80001802 80003802 80005802 80009802"
	$"80011802 80021802 8007F802 80081802"
	$"80101802 80201802 80401802 80801802"
	$"83C07E02 80000002 80000002 FFFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07F00000"
	$"0FF80000 1FFC0000 3FFE0000 7FFFFFFC"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	$"FFFFFFFE FFFFFFFE FFFFFFFE FFFFFFFE"
	} };

resource 'ics#' (fontsFolderIconResource, purgeable)
	{ {
	$"0000 0000 0000 3C00 4200 FFFE 8001 8021"
	$"8061 80A1 81E1 8221 8421 8E71 8001 FFFF";

	$"0000 0000 0000 3C00 7E00 FFFE FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	} };

resource 'icl8' (fontsFolderIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFF0000000000000000000000000000000000000000"
	$"00000000FF2A2A2A2A2A2A2AFF00000000000000000000000000000000000000"
	$"000000FF2A2A2A2A2A2A2A2A2AFF000000000000000000000000000000000000"
	$"0000FF2A2A2A2A2A2A2A2A2A2A2AFF0000000000000000000000000000000000"
	$"00FF2A2A2A2A2A2A2A2A2A2A2A2A2AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"FF2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2AFF00"
	$"FF0000000000000000000000000000000000000000000000000000000000FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002AB02A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002AB0B0002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002AB0B0B02A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002AB02AB0B0002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002AB02A00B0B02A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002AB02A002AB0B0002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002AB02A002A00B0B02A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002AB0B0B0B0B0B0B0B0002A002A002A002A54FF00"
	$"FF2A002A002A002A002A002AB02A002A002A00B0B02A002A002A002A0054FF00"
	$"FF002A002A002A002A002AB02A002A002A002AB0B0002A002A002A002A54FF00"
	$"FF2A002A002A002A002AB02A002A002A002A00B0B02A002A002A002A0054FF00"
	$"FF002A002A002A002AB02A002A002A002A002AB0B0002A002A002A002A54FF00"
	$"FF2A002A002A002AB02A002A002A002A002A00B0B02A002A002A002A0054FF00"
	$"FF002A002A00B0B0B0B02A002A002A002AB0B0B0B0B0B0002A002A002A54FF00"
	$"FF2A002A002A002A002A002A002A002A002A002A002A002A002A002A0054FF00"
	$"FF002A002A002A002A002A002A002A002A002A002A002A002A002A002A54FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'icl4' (fontsFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFF00000000000000000000"
	$"0000FCCCCCCCF0000000000000000000"
	$"000FCCCCCCCCCF000000000000000000"
	$"00FCCCCCCCCCCCF00000000000000000"
	$"0FCCCCCCCCCCCCCFFFFFFFFFFFFFFF00"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"F00000000000000000000000000000F0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C5C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C550C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C555C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C5C550C0C0C0CCF0"
	$"FC0C0C0C0C0C0C0C5C055C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C5C0C550C0C0C0CCF0"
	$"FC0C0C0C0C0C0C5C0C055C0C0C0C0CF0"
	$"F0C0C0C0C0C0C555555550C0C0C0CCF0"
	$"FC0C0C0C0C0C5C0C0C055C0C0C0C0CF0"
	$"F0C0C0C0C0C5C0C0C0C550C0C0C0CCF0"
	$"FC0C0C0C0C5C0C0C0C055C0C0C0C0CF0"
	$"F0C0C0C0C5C0C0C0C0C550C0C0C0CCF0"
	$"FC0C0C0C5C0C0C0C0C055C0C0C0C0CF0"
	$"F0C0C05555C0C0C0C5555550C0C0CCF0"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0"
	};

resource 'ics8' (fontsFolderIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000FFFFFFFF00000000000000000000"
	$"00FF2A2A2A2AFF000000000000000000"
	$"FF2A2A2A2A2A2AFFFFFFFFFFFFFFFF00"
	$"FF002A002A002A002A002A002A0054FF"
	$"FF2A002A002A002A002AB02A002A54FF"
	$"FF002A002A002A002AB0B0002A0054FF"
	$"FF2A002A002A002AB02AB02A002A54FF"
	$"FF002A002A002AB0B0B0B0002A0054FF"
	$"FF2A002A002AB02A002AB02A002A54FF"
	$"FF002A002AB02A002A00B0002A0054FF"
	$"FF2A002AB0B0B02A00B0B0B0002A54FF"
	$"FF002A002A002A002A002A002A0054FF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (fontsFolderIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00FFFF0000000000" 
	$"0FCCCCF000000000"
	$"FCCCCCCFFFFFFFF0"
	$"F0C0C0C0C0C0C0CF"
	$"FC0C0C0C0C5C0CCF"
	$"F0C0C0C0C550C0CF"
	$"FC0C0C0C5C5C0CCF"
	$"F0C0C0C55550C0CF"
	$"FC0C0C5C0C5C0CCF"
	$"F0C0C5C0C050C0CF"
	$"FC0C555C05550CCF"
	$"F0C0C0C0C0C0C0CF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// generic desk accessory

// Which of these two is correct? I chose the one I got from Bryan Stearns because it
// contains a mask which may be needed by the Finder.
/*
resource 'SICN' (genericDeskAccessoryIconResource, purgeable)
	{ {
		$"0000 0000 0080 0140 0220 0410 0B08 1484"
		$"0BC8 1910 1E20 1940 0080 0000 0000 0000";
	} };
*/

resource 'SICN' (genericDeskAccessoryIconResource, purgeable)
	{ {
	$"0000 0000 0100 0280 0440 0820 1610 2908"
	$"1790 3A20 3C40 3280 0100 0000 0000 0000";

	$"0000 0000 0100 0380 07C0 0FE0 1FF0 3FF8"
	$"1FF0 3FE0 3FC0 3F80 0100 0000 0000 0000"
	} };

resource 'ICN#' (genericDeskAccessoryIconResource, purgeable)
	{ {
	$"00008000 00014000 00022000 00041000"
	$"00080800 00100400 00200200 00400100"
	$"00800080 01000040 02000020 04000010"
	$"08000008 10000004 20FC0002 41020001"
	$"82010002 440C8004 28138008 F1FE7010"
	$"E00C4020 E0008040 E0010080 E0060100"
	$"E7F80200 F8400400 E0200800 00101000"
	$"00082000 00044000 00028000 00010000";

	$"00008000 0001C000 0003E000 0007F000"
	$"000FF800 001FFC00 003FFE00 007FFF00"
	$"00FFFF80 01FFFFC0 03FFFFE0 07FFFFF0"
	$"0FFFFFF8 1FFFFFFC 3FFFFFFE 7FFFFFFF"
	$"FFFFFFFE 7FFFFFFC 3FFFFFF8 FFFFFFF0"
	$"FFFFFFE0 FFFFFFC0 FFFFFF80 FFFFFF00"
	$"FFFFFE00 F87FFC00 E03FF800 001FF000"
	$"000FE000 0007C000 00038000 00010000"
	} };

resource 'ics#' (genericDeskAccessoryIconResource, purgeable)
	{ {
	$"0000 0080 0140 0220 0410 0808 1004 2602"
	$"4901 7282 6F64 6088 7F10 6220 0140 0080";

	$"0000 0080 01C0 03E0 07F0 0FF8 1FFC 3FFE"
	$"7FFF 7FFE 7FFC 7FF8 7FF0 63E0 01C0 0080"
	} };

resource 'icl8' (genericDeskAccessoryIconResource, purgeable)
	{
	$"00000000000000000000000000000000FF000000000000000000000000000000"
	$"000000000000000000000000000000FFF5FF0000000000000000000000000000"
	$"0000000000000000000000000000FFF5F5F5FF00000000000000000000000000"
	$"00000000000000000000000000FFF5F5F5F5F5FF000000000000000000000000"
	$"000000000000000000000000FFF5F5F5F5F5F5F5FF0000000000000000000000"
	$"0000000000000000000000FFF5F5F5F5F5F5F5F5F5FF00000000000000000000"
	$"00000000000000000000FFF5F5F5F5F5F5F5F5F5F5F5FF000000000000000000"
	$"000000000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF0000000000000000"
	$"0000000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000000000"
	$"00000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF000000000000"
	$"000000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000000000"
	$"0000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"00000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5F5F5F5F5FFFFFFFFFFFFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00"
	$"00FFF5F5F5F5F5FF080808080808FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF"
	$"FFF5F5F5F5F5FF0808080808080808FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00"
	$"00FFF5F5F5FF080808080808FFFF0808FFF5F5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5FF080808080808FFF5F5FFFFFFF5F5F5F5F5F5F5F5F5F5F5FF000000"
	$"FFFFFFFF080808FFFFFFFFFFFFFFFF0808FFFFFFF5F5F5F5F5F5F5FF00000000"
	$"FFFFFF080808080808080808FFFF080808FFF5F5F5F5F5F5F5F5FF0000000000"
	$"FFFFFF08080808080808080808080808FFF5F5F5F5F5F5F5F5FF000000000000"
	$"FFFFFF080808080808080808080808FFF5F5F5F5F5F5F5F5FF00000000000000"
	$"FFFFFF08080808080808080808FFFFF5F5F5F5F5F5F5F5FF0000000000000000"
	$"FFFFFF0808FFFFFFFFFFFFFFFFF5F5F5F5F5F5F5F5F5FF000000000000000000"
	$"FFFFFFFFFF00000000FFF5F5F5F5F5F5F5F5F5F5F5FF00000000000000000000"
	$"FFFFFF00000000000000FFF5F5F5F5F5F5F5F5F5FF0000000000000000000000"
	$"0000000000000000000000FFF5F5F5F5F5F5F5FF000000000000000000000000"
	$"000000000000000000000000FFF5F5F5F5F5FF00000000000000000000000000"
	$"00000000000000000000000000FFF5F5F5FF0000000000000000000000000000"
	$"0000000000000000000000000000FFF5FF000000000000000000000000000000"
	$"000000000000000000000000000000FF00000000000000000000000000000000"
	};

resource 'icl4' (genericDeskAccessoryIconResource, purgeable)
	{
	$"0000000000000000F000000000000000"
	$"000000000000000F0F00000000000000"
	$"00000000000000F0C0F0000000000000"
	$"0000000000000F0C0C0F000000000000"
	$"000000000000F0C0C0C0F00000000000"
	$"00000000000F0C0C0C0C0F0000000000"
	$"0000000000F0C0C0C0C0C0F000000000"
	$"000000000F0C0C0C0C0C0C0F00000000"
	$"00000000F0C0C0C0C0C0C0C0F0000000"
	$"0000000F0C0C0C0C0C0C0C0C0F000000"
	$"000000F0C0C0C0C0C0C0C0C0C0F00000"
	$"00000F0C0C0C0C0C0C0C0C0C0C0F0000"
	$"0000F0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0C0F00"
	$"00F0C0C0FFFFFFC0C0C0C0C0C0C0C0F0"
	$"0F0C0C0F020202FC0C0C0C0C0C0C0C0F"
	$"F0C0C0F02020202FC0C0C0C0C0C0C0F0"
	$"0F0C0F020202FF02FC0C0C0C0C0C0F00"
	$"00F0F020202FC0FFF0C0C0C0C0C0F000"
	$"FFFF020FFFFFFFF20FFF0C0C0C0F0000"
	$"FFF020202020FF202FC0C0C0C0F00000"
	$"FFF2020202020202FC0C0C0C0F000000"
	$"FFF020202020202FC0C0C0C0F0000000"
	$"FFF2020202020FFC0C0C0C0F00000000"
	$"FFF02FFFFFFFF0C0C0C0C0F000000000"
	$"FFFFF0000F0C0C0C0C0C0F0000000000"
	$"FFF0000000F0C0C0C0C0F00000000000"
	$"00000000000F0C0C0C0F000000000000"
	$"000000000000F0C0C0F0000000000000"
	$"0000000000000F0C0F00000000000000"
	$"00000000000000F0F000000000000000"
	$"000000000000000F0000000000000000"
	};

resource 'ics8' (genericDeskAccessoryIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"0000000000000000FF00000000000000"
	$"00000000000000FFF5FF000000000000"
	$"000000000000FFF5F5F5FF0000000000"
	$"0000000000FFF5F5F5F5F5FF00000000"
	$"00000000FFF5F5F5F5F5F5F5FF000000"
	$"000000FFF5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5F5FFFFF5F5F5F5F5F5F5FF00"
	$"00FFF5F5FF0808FFF5F5F5F5F5F5F5FF"
	$"00FFFFFF0808FF08FFF5F5F5F5F5FF00"
	$"00FFFF08FFFFFFFF08FFFFF5F5FF0000"
	$"00FFFF0808080808FFF5F5F5FF000000"
	$"00FFFFFFFFFFFFFFF5F5F5FF00000000"
	$"00FFFF000000FFF5F5F5FF0000000000"
	$"00000000000000FFF5FF000000000000"
	$"0000000000000000FF00000000000000"
	};

resource 'ics4' (genericDeskAccessoryIconResource, purgeable)
	{
	$"0000000000000000"
	$"00000000F0000000"
	$"0000000FCF000000"
	$"000000FC0CF00000"
	$"00000FC0C0CF0000"
	$"0000FC0C0C0CF000"
	$"000FC0C0C0C0CF00"
	$"00FC0FFC0C0C0CF0"
	$"0FC0F02FC0C0C0CF"
	$"0FFF02F0FC0C0CF0"
	$"0FF0FFFF2FF0CF00"
	$"0FF20202FC0CF000"
	$"0FFFFFFFC0CF0000"
	$"0FF000FC0CF00000"
	$"0000000FCF000000"
	$"00000000F0000000"
	};

//__________________________________________________________________________________________________
// generic edition file

resource 'ICN#' (genericEditionFileIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 55555554 2AAAAAA8"
	$"40000004 20000008 47BC7004 20008808"
	$"47790404 20010408 47B907C4 20008848"
	$"47807044 20101048 40281044 20441048"
	$"40821FC4 21010008 43FF8004 20000008"
	$"40000004 2AAAAAA8 55555554 00000000";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 FFFFFFFE D5555556 BFFFFFFA"
	$"FFFFFFFE BFFFFFFA FFFFFFFE BFFFFFFA"
	$"FFFFFFFE BFFFFFFA FFFFFFFE BFFFFFFA"
	$"FFFFFFFE BFFFFFFA FFFFFFFE BFFFFFFA"
	$"FFFFFFFE BFFFFFFA FFFFFFFE BFFFFFFA"
	$"FFFFFFFE BFFFFFFA D5555556 FFFFFFFE"
	} };

resource 'ics#' (genericEditionFileIconResource, purgeable)
	{ {
	$"0000 0000 0000 5555 AAAA 4001 807A 4049"
	$"804A 4279 8502 4881 9FC2 4001 AAAA 5555";

	$"0000 0000 0000 5555 FFFE 7FFF FFFE 7FFF"
	$"FFFE 7FFF FFFE 7FFF FFFE 7FFF FFFE 5555"
	} };

resource 'icl8' (genericEditionFileIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF0000"
	$"0000FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FF000000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5FFFFFFFFF5FFFFFFFFF5F5F5FFFFFFF5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF7F7F7FFFF5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5FFFFFFF5FFFFFFFFF5F5FF7F7F7F7F7FFFF5F5F5F5F5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5FF7F7F7F7F7FFFF5F5F5F5F5F5FF000000"
	$"00FFF5F5F5FFFFFFFFF5FFFFFFF5F5FF7F7F7F7F7FFFFFFFFFFFF5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF7F7F7FFF92929292FFF5F5FF000000"
	$"00FFF5F5F5FFFFFFFFF5F5F5F5F5F5F5F5FFFFFF9292929292FFF5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5F5F5FFF5F5F5F5F5F5F5FF9292929292FFF5F5FF000000"
	$"00FFF5F5F5F5F5F5F5F5FFC0FFF5F5F5F5F5F5FF9292929292FFF5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5FFC0C0C0FFF5F5F5F5F5FF9292929292FFF5F5FF000000"
	$"00FFF5F5F5F5F5F5FFC0C0C0C0C0FFF5F5F5F5FFFFFFFFFFFFFFF5F5F5FF0000"
	$"0000FFF5F5F5F5FFC0C0C0C0C0C0C0FFF5F5F5F5F5F5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5F5FFFFFFFFFFFFFFFFFFFFFFF5F5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"0000FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FFF5FF000000"
	$"00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF00FF0000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	};

resource 'icl4' (genericEditionFileIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F00"
	$"00F0F0F0F0F0F0F0F0F0F0F0F0F0F000"
	$"0F0C0C0C0C0C0C0C0C0C0C0C0C0C0F00"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"0F0C0FFFFCFFFF0C0FFF0C0C0C0C0F00"
	$"00F0C0C0C0C0C0C0F5D5F0C0C0C0F000"
	$"0F0C0FFF0FFFFC0F5D5D5F0C0C0C0F00"
	$"00F0C0C0C0C0C0CFD5D5DFC0C0C0F000"
	$"0F0C0FFFFCFFFC0F5D5D5FFFFF0C0F00"
	$"00F0C0C0C0C0C0C0F5D5F8080FC0F000"
	$"0F0C0FFFFC0C0C0C0FFF80808F0C0F00"
	$"00F0C0C0C0CFC0C0C0CF08080FC0F000"
	$"0F0C0C0C0CF7FC0C0C0F80808F0C0F00"
	$"00F0C0C0CF777FC0C0CF08080FC0F000"
	$"0F0C0C0CF77777FC0C0FFFFFFF0C0F00"
	$"00F0C0CF7777777FC0C0C0C0C0C0F000"
	$"0F0C0CFFFFFFFFFFFC0C0C0C0C0C0F00"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"0F0C0C0C0C0C0C0C0C0C0C0C0C0C0F00"
	$"00F0F0F0F0F0F0F0F0F0F0F0F0F0F000"
	$"0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F00"
	$"00000000000000000000000000000000"
	};

resource 'ics8' (genericEditionFileIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00FF00FF00FF00FF00FF00FF00FF00FF"
	$"FF00FFF5FFF5FFF5FFF5FFF5FFF5FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF"
	$"FFF5F5F5F5F5F5F5F5FFFFFFFFF5FF00"
	$"00FFF5F5F5F5F5F5F5FF9292FFF5F5FF"
	$"FFF5F5F5F5F5F5F5F5FF9292FFF5FF00"
	$"00FFF5F5F5F5FFF5F5FFFFFFFFF5F5FF"
	$"FFF5F5F5F5FFC0FFF5F5F5F5F5F5FF00"
	$"00FFF5F5FFC0C0C0FFF5F5F5F5F5F5FF"
	$"FFF5F5FFFFFFFFFFFFFFF5F5F5F5FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF"
	$"FF00FFF5FFF5FFF5FFF5FFF5FFF5FF00"
	$"00FF00FF00FF00FF00FF00FF00FF00FF"
	};

resource 'ics4' (genericEditionFileIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0F0F0F0F0F0F0F0F"
	$"F0F0F0F0F0F0F0F0"
	$"0F0C0C0C0C0C0C0F"
	$"F0C0C0C0CFFFF0F0"
	$"0F0C0C0C0F08FC0F"
	$"F0C0C0C0CF80F0F0"
	$"0F0C0CFC0FFFFC0F"
	$"F0C0CF7FC0C0C0F0"
	$"0F0CF777FC0C0C0F"
	$"F0CFFFFFFFC0C0F0"
	$"0F0C0C0C0C0C0C0F"
	$"F0F0F0F0F0F0F0F0"
	$"0F0F0F0F0F0F0F0F"
	};

resource 'SICN' (genericEditionFileIconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 1550 2AA8 1010 2008"
	$"1010 2008 1010 2AA8 1550 0000 0000 0000";

	$"0000 0000 0000 0000 3FF8 3FF8 3FF8 3FF8"
	$"3FF8 3FF8 3FF8 3FF8 3FF8 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic stationery pad

resource 'ICN#' (genericStationeryIconResource, purgeable)
	{ {
	$"3FFFFFE0 20000020 20000038 20000028"
	$"20000028 20000028 20000028 20000028"
	$"20000028 20000028 20000028 20000028"
	$"20000028 20000028 20000028 20000028"
	$"20000028 20000028 20000028 20000028"
	$"20000028 20000028 20000028 20000FE8"
	$"20000848 20000888 20000908 20000A08"
	$"20000C08 3FFFF808 08000008 0FFFFFF8";

	$"3FFFFFE0 3FFFFFE0 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 3FFFFFF8 3FFFFFF8"
	$"3FFFFFF8 3FFFFFF8 0FFFFFF8 0FFFFFF8"
	} };

resource 'ics#' (genericStationeryIconResource, purgeable)
	{ {
	$"7FF8 4008 400E 400A 400A 400A 400A 400A"
	$"400A 400A 407A 4052 4062 7FC2 1002 1FFE";

	$"7FF8 7FF8 7FFE 7FFE 7FFE 7FFE 7FFE 7FFE"
	$"7FFE 7FFE 7FFE 7FFE 7FFE 7FFE 1FFE 1FFE"
	} };

resource 'icl8' (genericStationeryIconResource, purgeable)
	{
	$"0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF0000000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFFF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFFFFFFFFFFFF9FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2B2BF7FFF9F7FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2BF7FFF9F72BFF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2BF7FFF9F72BF6FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFF7FFF9F72BF6F6FF000000"
	$"0000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFF9F72BF6F6F6FF000000"
	$"0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF9F72BF6F6F6F6FF000000"
	$"00000000FFF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F72BF6F6F6F6F6FF000000"
	$"00000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'icl4' (genericStationeryIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFFFFFFF00000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CF00000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FFF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0C0C0C0CFDF000"
	$"00F0C0C0C0C0C0C0C0C0C0C0C0FDF000"
	$"00FC0C0C0C0C0C0C0C0CFFFFFFFDF000"
	$"00F0C0C0C0C0C0C0C0C0FCCCCFDDF000"
	$"00FC0C0C0C0C0C0C0C0CFCCCFDDCF000"
	$"00F0C0C0C0C0C0C0C0C0FCCFDDCCF000"
	$"00FC0C0C0C0C0C0C0C0CFCFDDCCCF000"
	$"00F0C0C0C0C0C0C0C0C0FFDDCCCCF000"
	$"00FFFFFFFFFFFFFFFFFFFDDCCCCCF000"
	$"0000FDDDDDDDDDDDDDDDDDCCCCCCF000"
	$"0000FFFFFFFFFFFFFFFFFFFFFFFFF000"
	};

resource 'ics8' (genericStationeryIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFFFFFF000000"
	$"00FFF5F5F5F5F5F5F5F5F5F5FF000000"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFFFFF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5FFF9FF00"
	$"00FFF5F5F5F5F5F5F5FFFFFFFFF9FF00"
	$"00FFF5F5F5F5F5F5F5FF2BFFF9F9FF00"
	$"00FFF5F5F5F5F5F5F5FFFFF9F9F5FF00"
	$"00FFFFFFFFFFFFFFFFFFF9F9F5F5FF00"
	$"000000FFF9F9F9F9F9F9F9F5F5F5FF00"
	$"000000FFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'ics4' (genericStationeryIconResource, purgeable)
	{
	$"0FFFFFFFFFFFF000"
	$"0FC0C0C0C0C0F000"
	$"0F0C0C0C0C0CFFF0"
	$"0FC0C0C0C0C0FDF0"
	$"0F0C0C0C0C0CFDF0"
	$"0FC0C0C0C0C0FDF0"
	$"0F0C0C0C0C0CFDF0"
	$"0FC0C0C0C0C0FDF0"
	$"0F0C0C0C0C0CFDF0"
	$"0FC0C0C0C0C0FDF0"
	$"0F0C0C0C0FFFFDF0"
	$"0FC0C0C0CFCFDDF0"
	$"0F0C0C0C0FFDDCF0"
	$"0FFFFFFFFFDDC0F0"
	$"000FDDDDDDDC0CF0"
	$"000FFFFFFFFFFFF0"
	};

resource 'SICN' (genericStationeryIconResource, purgeable)
	{ {
	$"0000 0000 0000 0FF0 0810 081C 0814 0814"
	$"0814 0814 0874 0864 0FC4 0204 03FC 0000"
	} };

//__________________________________________________________________________________________________
// generic fileserver

resource 'SICN' (genericFileServerIconResource, purgeable)
	{ {
	$"0000 0000 0CF8 1788 10C8 10A8 10C8 3FDC"
	$"1510 0A90 0410 0390 00F0 00F0 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic preferences file

resource 'ICN#' (genericPreferencesIconResource, purgeable)
	{ {
	$"1FFFF800 10000C00 10000A00 10000900"
	$"10000880 10000840 10000820 10000FF0"
	$"10000010 10E00010 11100010 12E80010"
	$"12E9F790 12E80010 11100010 10E00010"
	$"10000010 10000010 10000010 10000010"
	$"10000010 10E00010 11100010 12080010"
	$"1209EF90 12080010 11100010 10E00010"
	$"10000010 10000010 10000010 1FFFFFF0";

	$"1FFFF800 1FFFFC00 1FFFFE00 1FFFFF00"
	$"1FFFFF80 1FFFFFC0 1FFFFFE0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	} };

resource 'ics#' (genericPreferencesIconResource, purgeable)
	{ {
	$"7FE0 4030 4028 403C 4604 4F04 4F74 4604"
	$"4004 4004 4604 4904 4974 4604 4004 7FFC";

	$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
	$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	} };

resource 'icl4' (genericPreferencesIconResource, purgeable)
	{
	$"000FFFFFFFFFFFFFFFFFF00000000000"
	$"000F0C0C0C0C0C0C0C0CFF0000000000"
	$"000FC0C0C0C0C0C0C0C0FCF000000000"
	$"000F0C0C0C0C0C0C0C0CFCCF00000000"
	$"000FC0C0C0C0C0C0C0C0FCCCF0000000"
	$"000F0C0C0C0C0C0C0C0CFCCCCF000000"
	$"000FC0C0C0C0C0C0C0C0FCCCCCF00000"
	$"000F0C0C0C0C0C0C0C0CFFFFFFFF0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0CFFFC0C0C0C0C0C0C0C0F0000"
	$"000FC0CF000FC0C0C0C0C0C0C0CF0000"
	$"000F0CF0FFF0FC0C0C0C0C0C0C0F0000"
	$"000FC0F0FFF0F0CFFFFFCFFFF0CF0000"
	$"000F0CF0FFF0FC0C0C0C0C0C0C0F0000"
	$"000FC0CF000FC0C0C0C0C0C0C0CF0000"
	$"000F0C0CFFFC0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0CFFFC0C0C0C0C0C0C0C0F0000"
	$"000FC0CF000FC0C0C0C0C0C0C0CF0000"
	$"000F0CF00000FC0C0C0C0C0C0C0F0000"
	$"000FC0F00000F0CFFFF0FFFFF0CF0000"
	$"000F0CF00000FC0C0C0C0C0C0C0F0000"
	$"000FC0CF000FC0C0C0C0C0C0C0CF0000"
	$"000F0C0CFFFC0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000FFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'icl8' (genericPreferencesIconResource, purgeable)
	{
	$"000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFF00000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2BFF000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2BFF0000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2B2BFF00000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2B2B2BFF000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2B2B2B2B2BFF0000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFFFFFFFFFFFFF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5FFFFFFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FF000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FF00FFFFFF00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FF00FFFFFF00FFF5F5FFFFFFFFFFF5FFFFFFFFF5F5FF00000000"
	$"000000FFF5F5FF00FFFFFF00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FF000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5FFFFFFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5FFFFFFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FF000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FF0000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FF0000000000FFF5F5FFFFFFFFF5FFFFFFFFFFF5F5FF00000000"
	$"000000FFF5F5FF0000000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FF000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5FFFFFFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	};

resource 'ics4' (genericPreferencesIconResource, purgeable)
	{
	$"0FFFFFFFFFF00000"
	$"0F0C0C0C0CFF0000"
	$"0FC0C0C0C0FCF000"
	$"0F0C0C0C0CFFFF00"
	$"0FC0CFF0C0C0CF00"
	$"0F0CFFFF0C0C0F00"
	$"0FC0FFFFCFFFCF00"
	$"0F0C0FFC0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0CFF0C0C0CF00"
	$"0F0CF00F0C0C0F00"
	$"0FC0F00FCFFFCF00"
	$"0F0C0FFC0C0C0F00"
	$"0FC0C0C0C0C0CF00"
	$"0FFFFFFFFFFFFF00"
	};

resource 'ics8' (genericPreferencesIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFF0000000000"
	$"00FFF5F5F5F5F5F5F5F5FFFF00000000"
	$"00FFF5F5F5F5F5F5F5F5FF2BFF000000"
	$"00FFF5F5F5F5F5F5F5F5FFFFFFFF0000"
	$"00FFF5F5F5FFFFF5F5F5F5F5F5FF0000"
	$"00FFF5F5FFFFFFFFF5F5F5F5F5FF0000"
	$"00FFF5F5FFFFFFFFF5FFFFFFF5FF0000"
	$"00FFF5F5F5FFFFF5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5FFFFF5F5F5F5F5F5FF0000"
	$"00FFF5F5FF0000FFF5F5F5F5F5FF0000"
	$"00FFF5F5FF0000FFF5FFFFFFF5FF0000"
	$"00FFF5F5F5FFFFF5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

//__________________________________________________________________________________________________
// generic Suitcase

resource 'SICN' (genericSuitcaseIconResource, purgeable)
	{ {
	$"0000 0000 0000 03C0 1FFC 3FFC 200C 200C"
	$"200C 200C 200C 200C 3FF8 0000 0000 0000";

	$"0000 0000 0000 03C0 1FFC 3FFC 3FFC 3FFC"
	$"3FFC 3FFC 3FFC 3FFC 3FF8 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic Mover object

resource 'SICN' (genericMoverObjectIconResource, purgeable)
	{ {
	$"0000 0000 0000 03F0 0610 0E10 0810 0810"
	$"0810 0810 0810 0810 0FF0 0000 0000 0000";

	$"0000 0000 0000 03F0 07F0 0FF0 0FF0 0FF0"
	$"0FF0 0FF0 0FF0 0FF0 0FF0 0000 0000 0000"
	} };

//__________________________________________________________________________________________________
// generic query document

resource 'ICN#' (genericQueryDocumentIconResource, purgeable)
	{ {
	$"1FFFF800 10000C00 10000A00 11FE0900"
	$"12010880 12FD0840 12850820 12850FF0"
	$"12850090 12FD0110 12013010 12010810"
	$"121D0410 12010410 11FE0410 11FE1F10"
	$"10008E10 10010410 11020010 13840010"
	$"17C87C10 11038390 11020090 10838790"
	$"10727C90 10038390 12027C90 14018310"
	$"18007C10 10000010 10000010 1FFFFFF0";

	$"1FFFF800 1FFFFC00 1FFFFE00 1FFFFF00"
	$"1FFFFF80 1FFFFFC0 1FFFFFE0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	$"1FFFFFF0 1FFFFFF0 1FFFFFF0 1FFFFFF0"
	} };

resource 'ics#' (genericQueryDocumentIconResource, purgeable)
	{ {
	$"7FE0 4030 5FA8 50BC 5684 5484 5084 4F04"
	$"4004 41E4 4214 43F4 43F4 41E4 4004 7FFC";

	$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
	$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	} };

resource 'icl8' (genericQueryDocumentIconResource, purgeable)
	{
	$"000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFF000000000000000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF2BFF0000000000000000"
	$"000000FFF5F5F5FFFFFFFFFFFFFFFFF5F5F5F5F5F5FF2B2BFF00000000000000"
	$"000000FFF5F5FF2B2B2B2B2B2B2B2BFFF5F5F5F5F5FF2B2B2BFF000000000000"
	$"000000FFF5F5FF2BFCFCFCFCFCFC2BFFF5F5F5F5F5FF2B2B2B2BFF0000000000"
	$"000000FFF5F5FF2BFC2A2A2A2AF52BFFF5F5F5F5F5FFFFFFFFFFFFFF00000000"
	$"000000FFF5F5FF2BFC2A2A2A2AF52BFFF5F5F5F5F5F5F5F5F5FFF5FF00000000"
	$"000000FFF5F5FF2BFC2A2A2A2AF52BFFF5F5F5F5F5F5F5F5FFF5F5FF00000000"
	$"000000FFF5F5FF2BFCF5F5F5F5F52BFFF5F5F5F5F5F5F5FFF5F5F5FF00000000"
	$"000000FFF5F5FF2B2B2B2B2B2B2B2BFFF5F5FFFFF5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FF2BE32B2B2B2B2B2BFFF5F5F5F5FFF5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FF2BD82B2BFFFFFF2BFFF5F5F5F5F5FFF5F5F5F5F5FF00000000"
	$"000000FFF5F5FF2B2B2B2B2B2B2B2BFFF5F5F5F5F5FFF5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FFFFFFFFFFFFFFFFF5F5F5F5F5F5FFF5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FFFFFFFFFFFFFFFFF5F5F5F5FFFFFFFFFFF5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5FFF5F5F5FFFFFFF5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5FFF5F5F5F5F5FFF5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FFF5F5F5F5F5F5FFF5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5FFFFFFF5F5F5F5FFF5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5FFFFFFFFFFF5F5FFF5F5F5F5FFFFFFFFFFF5F5F5F5F5FF00000000"
	$"000000FFF5F5F5FFF5F5F5F5F5F5FFFFFF2AF52AF52AFFFFFFF5F5FF00000000"
	$"000000FFF5F5F5FFF5F5F5F5F5F5FFF52AF52AF52AF52AF5FFF5F5FF00000000"
	$"000000FFF5F5F5F5FFF5F5F5F5F5FFFFFF2AF52AF5FFFFFFFFF5F5FF00000000"
	$"000000FFF5F5F5F5F5FFFFFFF5F5FFF8F5FFFFFFFFFFFCFDFFF5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5FFFFFFF8F8FBFBFCFFFFFFF5F5FF00000000"
	$"000000FFF5F5FFF5F5F5F5F5F5F5FFF8F5FFFFFFFFFFFCFDFFF5F5FF00000000"
	$"000000FFF5FFF5F5F5F5F5F5F5F5F5FFFFF8F9FBFCFCFFFFF5F5F5FF00000000"
	$"000000FFFFF5F5F5F5F5F5F5F5F5F5F5F5FFFFFFFFFFF5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00000000"
	$"000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	};

resource 'icl4' (genericQueryDocumentIconResource, purgeable)
	{
	$"000FFFFFFFFFFFFFFFFFFF0000000000"
	$"000FC0C0C0C0C0C0C0C0CFF000000000"
	$"000F0C0C0C0C0C0C0C0C0FCF00000000"
	$"000FC00FFFFFFFF0C0C0CF0CF0000000"
	$"000F0CFCCCCCCCCF0C0C0FC0CF000000"
	$"000FC0FCEEEEEECFC0C0CF0C0CF00000"
	$"000F0CFCE0E0E0CF0C0C0FFFFFFF0000"
	$"000FC0FCEE0E0DCFC0C0C0C0CFCF0000"
	$"000F0CFCE0E0E0CF0C0C0C0CFC0F0000"
	$"000FC0FCED0D0DCFC0C0C0CFC0CF0000"
	$"000F0CFCCCCCCCCF0CFF0C0C0C0F0000"
	$"000FC0FC8CCCCCCFC0C0F0C0C0CF0000"
	$"000F0CFC3CCFFFCF0C0C0F0C0C0F0000"
	$"000FC0FCCCCCCCCFC0C0CFC0C0CF0000"
	$"000F0C0FFFFFFFF00C0C0F0C0C0F0000"
	$"000FC00FFFFFFFF0C0CFFFFFC0CF0000"
	$"000F0C0C0C0C0C0CFC0CFFFC0C0F0000"
	$"000FC0C0C0C0C0CFC0C0CFC0C0CF0000"
	$"000F0C0F0C0C0CFC0C0C0C0C0C0F0000"
	$"000FC0FFF0C0CFC0C0C0C0C0C0CF0000"
	$"000F0FFFFF0CFC0C0FFFFF0C0C0F0000"
	$"000FC0CFC0C0C0FFFC0C0CFFF0CF0000"
	$"000F0C0F0C0C0CF0C0C0C0C0FC0F0000"
	$"000FC0C0F0C0C0FFFC0C0FFFF0CF0000"
	$"000F0C0C0FFF0CFC0FFFFFEEFC0F0000"
	$"000FC0C0C0C0C0FFFCCEEEFFF0CF0000"
	$"000F0CFC0C0C0CFC0FFFFFEEFC0F0000"
	$"000FCFC0C0C0C0CFFCDEEEFFC0CF0000"
	$"000FFC0C0C0C0C0C0FFFFF0C0C0F0000"
	$"000FC0C0C0C0C0C0C0C0C0C0C0CF0000"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0F0000"
	$"000FFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics8' (genericQueryDocumentIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFF0000000000"
	$"00FFF5F5F5F5F5F5F5F5FFFF00000000"
	$"00FFF5FFFFFFFFFFFFF5FF2BFF000000"
	$"00FFF5FF2B2B2B2BFFF5FFFFFFFF0000"
	$"00FFF5FF2BFFFF2BFFF5F5F5F5FF0000"
	$"00FFF5FF2BFF2A2BFFF5F5F5F5FF0000"
	$"00FFF5FF2B2B2B2BFFF5F5F5F5FF0000"
	$"00FFF5F5FFFFFFFFF5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFF5F5F5F5F5FFFFFFFFF5F5FF0000"
	$"00FFF5F5F5F5FF002A002AFFF5FF0000"
	$"00FFF5F5F5F5FFFFFFFFFFFFF5FF0000"
	$"00FFF5F5F5F5FFFFFFFFFFFFF5FF0000"
	$"00FFF5F5F5F5F5FFFFFFFFF5F5FF0000"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5FF0000"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics4' (genericQueryDocumentIconResource, purgeable)
	{
	$"0FFFFFFFFFF00000"
	$"0FC0C0C0C0FF0000"
	$"0F0FFFFFFCFCF000"
	$"0FCFCCCCF0FFFF00"
	$"0F0FCFFCFC0C0F00"
	$"0FCFCFCCF0C0CF00"
	$"0F0FCCCCFC0C0F00"
	$"0FC0FFFFC0C0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FC0C0CFFFF0CF00"
	$"0F0C0CF0C0CF0F00"
	$"0FC0C0FFFFFFCF00"
	$"0F0C0CFFFFFF0F00"
	$"0FC0C0CFFFF0CF00"
	$"0F0C0C0C0C0C0F00"
	$"0FFFFFFFFFFFFF00"
	};

//__________________________________________________________________________________________________
// model icons
//__________________________________________________________________________________________________

//__________________________________________________________________________________________________
// 128, 512, 512Ke, Plus, Classic, Classic II

resource 'ICN#' (MacPlusClassicIconResource, purgeable)
	{ {
	$"00000000 07FFFFE0 08000010 08000010"
	$"08FFFF10 09000090 09555090 09000090"
	$"09550090 09000090 09500090 09000090"
	$"09500090 09000090 09540090 09000090"
	$"09000090 08FFFF10 08000010 08000010"
	$"08000010 08000010 0800FF10 08000010"
	$"08000010 08000010 08000010 07FFFFE0"
	$"04000020 04000020 04000020 07FFFFE0";

	$"00000000 07FFFFE0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	} };

resource 'ics#' (MacPlusClassicIconResource, purgeable)
	{ {
	$"0000 1FF8 2004 2FF4 2814 2A94 2814 2A14"
	$"2814 2FF4 2004 2004 20F4 2004 1FF8 1FF8";

	$"0000 1FF8 3FFC 3FFC 3FFC 3FFC 3FFC 3FFC"
	$"3FFC 3FFC 3FFC 3FFC 3FFC 3FFC 1FF8 1FF8"
	} };

resource 'icl8' (MacPlusClassicIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2BFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC2B2B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2B2B000000000000000000000000000000002B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2BFFFFFFFFFFFFFFFF2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B00000000000000002B2B2BFF00000000"
	$"00000000FF2B2BE3E32B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2BD8D82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"0000000000FFFBFBFBFBFBFBFCFCFCFCFCFDFDFDFDFDFDFDFDFDFF0000000000"
	$"0000000000FFFAFAFAFAFAFAFAFAFBFBFCFCFCFCFDFDFDFDFDFDFF0000000000"
	$"0000000000FFF9F9F9F9F9F9F9F9FAFAFAFBFBFCFCFCFCFDFDFDFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	};

resource 'icl4' (MacPlusClassicIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCEEEEEEEEEEEEEEEECCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCC0C0C0C0C0C0C0C0C0CCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCFFFFFFFFCCCF0000"
	$"0000FCCCCCCCCCCC0C0C0C0CCCCF0000"
	$"0000FCC88CCCCCCCCCCCCCCCCCCF0000"
	$"0000FCC33CCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"00000FEEEEEEEEEEEEFFFFFFFFF00000"
	$"00000FDDDDDDDDEEEEEEEEFFFFF00000"
	$"00000FDDDDDDDDDDDEEEEEEEEEF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	};

resource 'ics8' (MacPlusClassicIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BFCFCFCFCFCFCFCFC2BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFA000000000000002BFF0000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BE32B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BD82B2B2BFFFFFFFF2BFF0000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"000000FDFDFDFDFDFDFFFFFFFF000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'ics4' (MacPlusClassicIconResource, purgeable)
	{
	$"0000000000000000"
	$"000FFFFFFFFFF000"
	$"00FCCCCCCCCCCF00"
	$"00FCEEEEEEEECF00"
	$"00FCED0D0D0CCF00"
	$"00FCE0D0D0D0CF00"
	$"00FCED0D0D0CCF00"
	$"00FCE0D0D0D0CF00"
	$"00FCED0D0D0CCF00"
	$"00FCD0C0C0C0CF00"
	$"00FCCCCCCCCCCF00"
	$"00FC8CCCCCCCCF00"
	$"00FC3CCCFFFFCF00"
	$"00FCCCCCCCCCCF00"
	$"000FFFFFFFFFF000"
	$"000FFFFFFFFFF000"
	};

//__________________________________________________________________________________________________
// SE, SE/30

resource 'ICN#' (MacSEIconResource, purgeable)
	{ {
	$"00000000 07FFFFE0 08000010 08000010"
	$"08FFFF10 09000090 09555090 09000090"
	$"09550090 09000090 09500090 09000090"
	$"09500090 09000090 09540090 09000090"
	$"09000090 08FFFF10 08000010 08000010"
	$"0800FF10 08000010 08000010 08000010"
	$"0800FF10 08000010 08000010 07FFFFE0"
	$"04000020 04000020 04000020 07FFFFE0";

	$"00000000 07FFFFE0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	} };

resource 'ics#' (MacSEIconResource, purgeable)
	{ {
	$"1FF8 2004 2FF4 2814 2A94 2814 2A14 2814"
	$"2FF4 2004 2004 20F4 2004 1FF8 1008 1FF8";

	$"1FF8 3FFC 3FFC 3FFC 3FFC 3FFC 3FFC 3FFC"
	$"3FFC 3FFC 3FFC 3FFC 3FFC 1FF8 1FF8 1FF8"
	} };

resource 'icl8' (MacSEIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2BFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC2B2B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2B2B000000000000000000000000000000002B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2BFFFFFFFFFFFFFFFF2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2BF5F5F5F5F5F5F5F52B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2BE3E32B2B2B2B2B2B2BFFFFFFFFFFFFFFFF2B2B2BFF00000000"
	$"00000000FF2B2BD8D82B2B2B2B2B2B2BF5F5F5F5F5F5F5F52B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"0000000000FFFBFBFCFBFCFBFCFCFDFCFDFDFEFDFEFDFEFDFEFDFF0000000000"
	$"0000000000FFFAFAFBFAFCFAFBFAFDFBFDFCFDFCFEFDFEFDFEFDFF0000000000"
	$"0000000000FFF9F9FAF9FAF9FBF9FBFAFBFBFDFCFDFCFEFDFEFDFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	};

resource 'icl4' (MacSEIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCEEEEEEEEEEEEEEEECCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCC0C0C0C0C0C0C0C0C0CCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCFFFFFFFFCCCF0000"
	$"0000FCCCCCCCCCCC0C0C0C0CCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCC88CCCCCCCFFFFFFFFCCCF0000"
	$"0000FCC33CCCCCCC0C0C0C0CCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"00000FEEEEEEEEEEEEFFFFFFFFF00000"
	$"00000FDDEDEDEDEEEEFEFEFFFFF00000"
	$"00000FDDEDEDEDEDEEFEFEFEFEF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	};

resource 'ics8' (MacSEIconResource, purgeable)
	{
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BFCFCFCFCFCFCFCFC2BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFA000000000000002BFF0000"
	$"0000FF2B2B2B2B2BFFFFFFFF2BFF0000"
	$"0000FF2BE32B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BD82B2B2BFFFFFFFF2BFF0000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"000000FDFDFDFDFDFDFFFFFFFF000000"
	$"000000FFF9FAFAFBFCFCFDFDFF000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'ics4' (MacSEIconResource, purgeable)
	{
	$"000FFFFFFFFFF000"
	$"00FCCCCCCCCCCF00"
	$"00FCEEEEEEEECF00"
	$"00FCED0D0D0CCF00"
	$"00FCE0D0D0D0CF00"
	$"00FCED0D0D0CCF00"
	$"00FCE0D0D0D0CF00"
	$"00FCED0D0D0CCF00"
	$"00FCD0C0C0C0CF00"
	$"00FCCCCCFFFFCF00"
	$"00FC8CCCCCCCCF00"
	$"00FC3CCCFFFFCF00"
	$"00FCCCCCCCCCCF00"
	$"000FFFFFFFFFF000"
	$"000FDDDEEEFFF000"
	$"000FFFFFFFFFF000"
	};

//__________________________________________________________________________________________________
// Portable

resource 'ICN#' (MacPortableIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000010"
	$"00000028 0000004C 00000092 00000124"
	$"00000248 00000490 00000920 00001240"
	$"00002480 00004900 00009200 00012400"
	$"0001C800 00013000 00021FFE 00021001"
	$"00033001 0001FFFF 0F3E4001 10C04001"
	$"7FFFFFFF 800007F9 80000001 FFFFFFFF";
	
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000010"
	$"00000038 0000007C 000000FE 000001FC"
	$"000003F8 000007F0 00000FE0 00001FC0"
	$"00003F80 00007F00 0000FE00 0001FC00"
	$"0001F800 0001F000 0003FFFE 0003FFFF"
	$"0003FFFF 0001FFFF 0F3FFFFF 1FFFFFFF"
	$"7FFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	} };

resource 'ics#' (MacPortableIconResource, purgeable)
	{ {
	$"0000 0000 0006 0009 0012 0024 0048 00D0"
	$"0120 013F 01C1 377F 4C41 FFFF 8001 FFFF";

	$"0000 0000 0006 000F 001E 003C 0078 00F0"
	$"01E0 01FF 01FF 37FF 7FFF FFFF FFFF FFFF"
	} };

resource 'icl8' (MacPortableIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"000000000000000000000000000000000000000000000000000000FF00000000"
	$"0000000000000000000000000000000000000000000000000000FFF5FF000000"
	$"00000000000000000000000000000000000000000000000000FFF52BFFFF0000"
	$"000000000000000000000000000000000000000000000000FFF52BFFF52BFF00"
	$"0000000000000000000000000000000000000000000000FFF52BFFF52BFF0000"
	$"00000000000000000000000000000000000000000000FFF52BFFF52BFF000000"
	$"000000000000000000000000000000000000000000FFF52BFFF52BFF00000000"
	$"0000000000000000000000000000000000000000FFF52BFFF52BFF0000000000"
	$"00000000000000000000000000000000000000FFF52BFFF52BFF000000000000"
	$"000000000000000000000000000000000000FFF52BFFF52BFF00000000000000"
	$"0000000000000000000000000000000000FFF52BFFF52BFF0000000000000000"
	$"00000000000000000000000000000000FFF52BFFF52BFF000000000000000000"
	$"000000000000000000000000000000FFF52BFFF52BFF00000000000000000000"
	$"000000000000000000000000000000FFFFFFF52BFF0000000000000000000000"
	$"000000000000000000000000000000FFF5F5FFFF000000000000000000000000"
	$"0000000000000000000000000000FFF52B2BF5FFFFFFFFFFFFFFFFFFFFFFFF00"
	$"0000000000000000000000000000FF2B2B2BF5FFF5F5F5F5F5F5F5F5F5F5F5FF"
	$"0000000000000000000000000000FFF92B2BF9FF2B2B2B2B2B2B2B2B2B2B2BFF"
	$"000000000000000000000000000000FFFFFFFFFBFBFBFBFBFBFBFBFBFBFBFBFF"
	$"00000000FCF9FCFE0000FFFFFFFFFF2B2BFB2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"000000FFF8F52BF8FFFF2B2B2B2B2B2B2BFB2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"00FFFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF"
	$"FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FFFFFFFFFFFFFFFFF5F5FF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BF5F5F5F5F5F5F5F52B2BFF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'icl4' (MacPortableIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"000000000000000000000000000F0000"
	$"00000000000000000000000000F0F000"
	$"0000000000000000000000000F0CFF00"
	$"000000000000000000000000F0CF0CF0"
	$"00000000000000000000000F0CF0CF00"
	$"0000000000000000000000F0CF0CF000"
	$"000000000000000000000F0CF0CF0000"
	$"00000000000000000000F0CF0CF00000"
	$"0000000000000000000F0CF0CF000000"
	$"000000000000000000F0CF0CF0000000"
	$"00000000000000000F0CF0CF00000000"
	$"0000000000000000F0CF0CF000000000"
	$"000000000000000F0CF0CF0000000000"
	$"000000000000000FFF0CF00000000000"
	$"000000000000000F00FF000000000000"
	$"00000000000000F0CC0FFFFFFFFFFFF0"
	$"00000000000000FCCC0F0C0C0C0C0C0F"
	$"00000000000000FDCCDFCCCCCCCCCCCF"
	$"000000000000000FFFFEEEEEEEEEEEEF"
	$"0000EDEF00FFFFFCCECCCCCCCCCCCCCF"
	$"000FC0CCFFCCCCCCCECCCCCCCCCCCCCF"
	$"0FEEEEEEEEEEEEEEEEEEEEEEEEEEEEEF"
	$"FC0C0C0C0C0C0C0C0C0C0FFFFFFFF00F"
	$"FCCCCCCCCCCCCCCCCCCCC0C0C0C0CCCF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics8' (MacPortableIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000FFFF00"
	$"000000000000000000000000FF2B2BFF"
	$"0000000000000000000000FF2B2BFF00"
	$"00000000000000000000FF2B2BFF0000"
	$"000000000000000000FF2B2BFF000000"
	$"0000000000000000FFFF2BFF00000000"
	$"00000000000000FFF52BFF0000000000"
	$"00000000000000FF2B2BFFFFFFFFFFFF"
	$"00000000000000FFFFFF2B2B2B2B2BFF"
	$"0000FFFF00FFFFFF2BFBFBFBFBFBFBFF"
	$"00FF2B2BFFFF2B2B2BF92B2B2B2B2BFF"
	$"FFFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	};

resource 'ics4' (MacPortableIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000FF0"
	$"000000000000FCCF"
	$"00000000000FCCF0"
	$"0000000000FCCF00"
	$"000000000FCCF000"
	$"00000000FFCF0000"
	$"0000000F0CF00000"
	$"0000000FCCFFFFFF"
	$"0000000FFFCCCCCF"
	$"00FF0FFFCEEEEEEF"
	$"0FCCFFCCCDCCCCCF"
	$"FEEEEEEEEEEEEEEF"
	$"FCCCCCCCCCCCCCCF"
	$"FFFFFFFFFFFFFFFF"
	};

//__________________________________________________________________________________________________
// IIcx, IIci

resource 'ICN#' (MacIIcxIIciIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 7FFFFFFE 40000002 40000032"
	$"40000002 4FFFFFF2 40000002 4FFFFFF2"
	$"40000002 40000002 40000FF2 4C000002"
	$"40000002 40000032 40000002 7FFFFFFE";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	} };

resource 'icl8' (MacIIcxIIciIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"00FCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC00"
	$"00FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00"
	$"00FFF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FDFDF9F9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00"
	$"00FF2B2B8181818181818181818181818181818181818181818181812B2BFF00"
	$"00FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00"
	$"00FF2B2B8181818181818181818181818181818181818181818181812B2BFF00"
	$"00FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00"
	$"00FFF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FFFFFFFFFFFFFFFFF9F9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00"
	$"00FF2B2BE3E32B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00"
	$"00FF2B2BD8D82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00"
	$"00FFF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F90505F9F9FF00"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF00"
	$"00FCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC00"
	};

resource 'icl4' (MacIIcxIIciIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0EFFFFFFFFFFFFFFFFFFFFFFFFFFFFE0"
	$"0FCCCCCCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"0FDDDDDDDDDDDDDDDDDDDDDDDDEEDDF0"
	$"0FC0C0C0C0C0C0C0C0C0C0C0C0C0C0F0"
	$"0FCCDDDDDDDDDDDDDDDDDDDDDDDDCCF0"
	$"0FCC0C0C0C0C0C0C0C0C0C0C0C0CCCF0"
	$"0FCCDDDDDDDDDDDDDDDDDDDDDDDDCCF0"
	$"0FCCC0C0C0C0C0C0C0C0C0C0C0C0CCF0"
	$"0FDDDDDDDDDDDDDDDDDDFFFFFFFFDDF0"
	$"0FC0C0C0C0C0C0C0C0C0C0C0C0C0C0F0"
	$"0FCC88CCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"0FCC33CCCCCCCCCCCCCCCCCCCCCCCCF0"
	$"0FDDDDDDDDDDDDDDDDDDDDDDDD11DDF0"
	$"0FC0C0C0C0C0C0C0C0C0C0C0C0C0C0F0"
	$"0EFFFFFFFFFFFFFFFFFFFFFFFFFFFFE0"
	};

//__________________________________________________________________________________________________
// II, IIx, IIfx

resource 'ICN#' (MacIIIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 FFFFFFFF"
	$"80000001 80000001 903FC7F9 80000001"
	$"80000001 80000001 80000001 80000001"
	$"80000001 FFFFFFFF D5555555 FFFFFFFF";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	} };

resource 'icl8' (MacIIIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"FCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FFF9F90505F9F9F9F9F9FFFFFFFFFFFFFFFFF9F9F9FFFFFFFFFFFFFFFFF9F9FF"
	$"FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF"
	$"FFF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FF"
	$"FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FF"
	$"FF2B2BE3E32B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2B2BD8D82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FFFCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FCF9FF"
	$"FFFBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FBF8FF"
	$"FCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC"
	};

resource 'icl4' (MacIIIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"EFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FDD11DDDDDFFFFFFFFDDDFFFFFFFFDDF"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0C0F"
	$"FDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDF"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0C0C0F"
	$"FCC88CCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FCC33CCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCF"
	$"FEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDF"
	$"FECECECECECECECECECECECECECECECF"
	$"EFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE"
	};

//__________________________________________________________________________________________________
// Macintosh IIsi

resource 'ICN#' (MacIIsiIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07FFFFE0"
	$"04000020 04000020 04003F20 05000020"
	$"04000020 07FFFFE0 02000040 03FFFFC0";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 03FFFFC0 03FFFFC0"
	} };

resource 'ics#' (MacIIsiIconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 FFFF 8001 A03D 8001 FFFF 4002 7FFE";

	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 FFFF FFFF FFFF FFFF FFFF 7FFE 7FFE"
	} };

resource 'icl8' (MacIIsiIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"0000000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF0000000000"
	$"0000000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF0000000000"
	$"0000000000FF2BE3E32B2B2B2B2B2B2B2B2BFFFFFFFFFFFF2B2BFF0000000000"
	$"0000000000FF2BD8D82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF0000000000"
	$"0000000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"000000000000FFF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FF000000000000"
	$"000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000"
	};

resource 'icl4' (MacIIsiIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"00000FCCCCCCCCCCCCCCCCCCCCF00000"
	$"00000FCCCCCCCCCCCCCCCCCCCCF00000"
	$"00000FC88CCCCCCCCCFFFFFFCCF00000"
	$"00000FC33CCCCCCCCCCCCCCCCCF00000"
	$"00000FCCCCCCCCCCCCCCCCCCCCF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"000000FDDDDDDDDDDDDDDDDDDF000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'ics8' (MacIIsiIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2BD82B2B2B2B2B2B2BFFFFFFFF2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	$"00FFF9F9F9F9F9F9F9F9F9F9F9F9FF00"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'ics4' (MacIIsiIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"FFFFFFFFFFFFFFFF"
	$"FCCCCCCCCCCCCCCF"
	$"FC3CCCCCCCFFFFCF"
	$"FCCCCCCCCCCCCCCF"
	$"FFFFFFFFFFFFFFFF"
	$"0FDDDDDDDDDDDDF0"
	$"0FFFFFFFFFFFFFF0"
	};

//__________________________________________________________________________________________________
// Macintosh LC

resource 'ICN#' (MacLCIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 07FFFFE0 04000020"
	$"05003F20 04000020 07FFFFE0 03FFFFC0";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 03FFFFC0"
	} };

resource 'ics#' (MacLCIconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 FFFF 8001 A07D 8001 FFFF 7FFE";

	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF 7FFE"
	} };

resource 'icl8' (MacLCIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"0000000000FF2BE3E32B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF0000000000"
	$"0000000000FF2BD8D82B2B2B2B2B2B2B2B2BFFFFFFFFFFFF2B2BFF0000000000"
	$"0000000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000"
	};

resource 'icl4' (MacLCIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"00000FC88CCCCCCCCCCCCCCCCCF00000"
	$"00000FC33CCCCCCCCCFFFFFFCCF00000"
	$"00000FCCCCCCCCCCCCCCCCCCCCF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'ics8' (MacLCIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FF2BD82B2B2B2B2B2BFFFFFFFFFF2BFF"
	$"FF2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'ics4' (MacLCIconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"FFFFFFFFFFFFFFFF"
	$"FCCCCCCCCCCCCCCF"
	$"FC3CCCCCCFFFFFCF"
	$"FCCCCCCCCCCCCCCF"
	$"FFFFFFFFFFFFFFFF"
	$"0FFFFFFFFFFFFFF0"
	};

//__________________________________________________________________________________________________
// Powerbook 100

resource 'ICN#' (PowerBook100IconResource, purgeable) { {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000004"
	$"0000000A 00000012 00000024 00000048"
	$"00000090 00000120 00000240 00000480"
	$"00000900 00001200 00001400 006DBC00"
	$"00FFE400 1F000400 20000400 3FFFF800";
	
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000004"
	$"0000000E 0000001E 0000003C 00000078"
	$"000000F0 000001E0 000003C0 00000780"
	$"00000F00 00001E00 00001C00 006DBC00"
	$"00FFFC00 1FFFFC00 3FFFFC00 3FFFF800"
	} };

resource 'ics#' (PowerBook100IconResource, purgeable) { {
	$"0000 0000 0000 0000 0000 0000 0000 0003"
	$"0007 000E 001C 0038 0070 0FE0 7020 7FE0";
	
	$"0000 0000 0000 0000 0000 0000 0000 0003"
	$"0007 000E 001C 0038 0070 0FE0 7FE0 7FE0"
	} };

resource 'icl8' (PowerBook100IconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000FF0000"
	$"00000000000000000000000000000000000000000000000000000000FFF8FF00"
	$"000000000000000000000000000000000000000000000000000000FFF8F8FF00"
	$"0000000000000000000000000000000000000000000000000000FFF8F8FF0000"
	$"00000000000000000000000000000000000000000000000000FFF8F8FF000000"
	$"000000000000000000000000000000000000000000000000FFF8F8FF00000000"
	$"0000000000000000000000000000000000000000000000FFF8F8FF0000000000"
	$"00000000000000000000000000000000000000000000FFF8F8FF000000000000"
	$"000000000000000000000000000000000000000000FFF8F8FF00000000000000"
	$"0000000000000000000000000000000000000000FFF8F8FF0000000000000000"
	$"00000000000000000000000000000000000000FFF8F8FF000000000000000000"
	$"00000000000000000000000000000000000000FFF8FF00000000000000000000"
	$"000000000000000000FFFF00FFFF00FFFF00FFFFFFFF00000000000000000000"
	$"0000000000000000FFFFFFFFFFFFFFFFFFFFFBF8FBFF00000000000000000000"
	$"000000FFFFFFFFFFF8F8F8F8F8F8F8F8F8F8FBF8F8FF00000000000000000000"
	$"0000FFF8F8F8F8F8F8F8F8F8F8F8F8F8F8F8FBF8FBFF00000000000000000000"
	$"0000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000000000"
	};

resource 'icl4' (PowerBook100IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000F00"
	$"0000000000000000000000000000FCF0"
	$"000000000000000000000000000FCCF0"
	$"00000000000000000000000000FCCF00"
	$"0000000000000000000000000FCCF000"
	$"000000000000000000000000FCCF0000"
	$"00000000000000000000000FCCF00000"
	$"0000000000000000000000FCCF000000"
	$"000000000000000000000FCCF0000000"
	$"00000000000000000000FCCF00000000"
	$"0000000000000000000FCCF000000000"
	$"0000000000000000000FCF0000000000"
	$"000000000FF0FF0FF0FFFF0000000000"
	$"00000000FFFFFFFFFFECEF0000000000"
	$"000FFFFFCCCCCCCCCCECCF0000000000"
	$"00FCCCCCCCCCCCCCCCECEF0000000000"
	$"00FFFFFFFFFFFFFFFFFFF00000000000"
	};

resource 'ics4' (PowerBook100IconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"00000000000000FF"
	$"0000000000000FFF"
	$"000000000000FFF0"
	$"00000000000FFF00"
	$"0000000000FFF000"
	$"000000000FFF0000"
	$"0000FFFFFFF00000"
	$"0FFFCCCCCCF00000"
	$"0FFFFFFFFFF00000"
	};

resource 'ics8' (PowerBook100IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"0000000000000000000000000000FFFF"
	$"00000000000000000000000000FFFFFF"
	$"000000000000000000000000FFFFFF00"
	$"0000000000000000000000FFFFFF0000"
	$"00000000000000000000FFFFFF000000"
	$"000000000000000000FFFFFF00000000"
	$"00000000FFFFFFFFFFFFFF0000000000"
	$"00FFFFFFF8F8F8F8F8F8FF0000000000"
	$"00FFFFFFFFFFFFFFFFFFFF0000000000"
	};

//__________________________________________________________________________________________________
// PowerBook 140, PowerBook 170

resource 'ICN#' (PowerBook140170IconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000002 00000005 00000009"
	$"00000012 00000024 00000048 00000090"
	$"00000120 00000240 00000480 00000900"
	$"00000E00 00001C00 00DB7E00 01FFC200"
	$"7F004200 803F4200 80004200 FFFFFC00";
	
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000002 00000007 0000000F"
	$"0000001E 0000003C 00000078 000000F0"
	$"000001E0 000003C0 00000780 00000F00"
	$"00000E00 00001C00 00DB7E00 01FFFE00"
	$"7FFFFE00 FFFFFE00 FFFFFE00 FFFFFC00"
	} };

resource 'ics#' (PowerBook140170IconResource, purgeable)
	{ {
	$"0000 0000 0000 0000 0000 0000 0001 0003"
	$"0007 000E 001C 0038 0070 1FF0 F710 FFF0";
	
	$"0000 0000 0000 0000 0000 0000 0001 0003"
	$"0007 000E 001C 0038 0070 1FF0 FFF0 FFF0"
	} };

resource 'icl8' (PowerBook140170IconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"000000000000000000000000000000000000000000000000000000000000FF00"
	$"0000000000000000000000000000000000000000000000000000000000FFF8FF"
	$"00000000000000000000000000000000000000000000000000000000FFF8F8FF"
	$"000000000000000000000000000000000000000000000000000000FFF8F8FF00"
	$"0000000000000000000000000000000000000000000000000000FFF8F8FF0000"
	$"00000000000000000000000000000000000000000000000000FFF8F8FF000000"
	$"000000000000000000000000000000000000000000000000FFF8F8FF00000000"
	$"0000000000000000000000000000000000000000000000FFF8F8FF0000000000"
	$"00000000000000000000000000000000000000000000FFF8F8FF000000000000"
	$"000000000000000000000000000000000000000000FFF8F8FF00000000000000"
	$"0000000000000000000000000000000000000000FFF8F8FF0000000000000000"
	$"0000000000000000000000000000000000000000FFFFFF000000000000000000"
	$"00000000000000000000000000000000000000FFFFFF00000000000000000000"
	$"0000000000000000FFFF00FFFF00FFFF00FFFFFFFFFFFF000000000000000000"
	$"00000000000000FFFFFFFFFFFFFFFFFFFFFFFBF8F8FBFF000000000000000000"
	$"00FFFFFFFFFFFFFFF8F8F8F8F8F8F8F8F8FFF8F9FCF8FF000000000000000000"
	$"FFF8F8F8F8F8F8F8F8F8FFFFFFFFFFFFF8FFF8FCF9F8FF000000000000000000"
	$"FFF8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8FFFBF8F8FBFF000000000000000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000000000000000"
	};

resource 'icl4' (PowerBook140170IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"000000000000000000000000000000F0"
	$"00000000000000000000000000000FCF"
	$"0000000000000000000000000000FCCF"
	$"000000000000000000000000000FCCF0"
	$"00000000000000000000000000FCCF00"
	$"0000000000000000000000000FCCF000"
	$"000000000000000000000000FCCF0000"
	$"00000000000000000000000FCCF00000"
	$"0000000000000000000000FCCF000000"
	$"000000000000000000000FCCF0000000"
	$"00000000000000000000FCCF00000000"
	$"00000000000000000000FFF000000000"
	$"0000000000000000000FFF0000000000"
	$"00000000FF0FF0FF0FFFFFF000000000"
	$"0000000FFFFFFFFFFFECCEF000000000"
	$"0FFFFFFFCCCCCCCCCFCDECF000000000"
	$"FCCCCCCCCCFFFFFFCFCEDCF000000000"
	$"FCCCCCCCCCCCCCCCCFECCEF000000000"
	$"FFFFFFFFFFFFFFFFFFFFFF0000000000"
	};

resource 'ics8' (PowerBook140170IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"000000000000000000000000000000FF"
	$"0000000000000000000000000000FFFF"
	$"00000000000000000000000000FFFFFF"
	$"000000000000000000000000FFFFFF00"
	$"0000000000000000000000FFFFFF0000"
	$"00000000000000000000FFFFFF000000"
	$"000000000000000000FFFFFF00000000"
	$"000000FFFFFFFFFFFFFFFFFF00000000"
	$"FFFFFFFFF8FFFFFFFCFCFCFF00000000"
	$"FFFFFFFFFFFFFFFFFFFFFFFF00000000"
	};

resource 'ics4' (PowerBook140170IconResource, purgeable)
	{
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"0000000000000000"
	$"000000000000000F"
	$"00000000000000FF"
	$"0000000000000FFF"
	$"000000000000FFF0"
	$"00000000000FFF00"
	$"0000000000FFF000"
	$"000000000FFF0000"
	$"000FFFFFFFFF0000"
	$"FFFFCFFFEEEF0000"
	$"FFFFFFFFFFFF0000"
	};

//__________________________________________________________________________________________________
// Quadra 700

resource 'ICN#' (Quadra700IconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"007FFE00 00400200 00400200 005E8200"
	$"00408200 005E8200 00408200 005E8200"
	$"00408200 005E9A00 00409A00 005E0200"
	$"00400200 005E0200 00400200 005E0200"
	$"00400200 005E0200 00400200 005E0200"
	$"00400200 005E0200 00400200 005E0A00"
	$"00400200 005E0A00 00400200 007FFE00";

	$"00000000 00000000 00000000 00000000"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	$"007FFE00 007FFE00 007FFE00 007FFE00"
	} };

resource 'ics#' (Quadra700IconResource, purgeable)
	{ {
	$"0000 0FF0 0810 0F90 0890 0F90 08B0 0F10"
	$"0810 0F10 0810 0F10 0850 0F50 0810 0FF0";

	$"0000 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0"
	$"0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0"
	} };

resource 'icl8' (Quadra700IconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"000000000000000000FCFFFFFFFFFFFFFFFFFFFFFFFFFC000000000000000000"
	$"000000000000000000FF2B2B2B2B2B2B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B2B2B2B2B2B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812BFF2B2B2B052BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52BFF2B2B2B052BFF000000000000000000"
	$"000000000000000000FF2B818181812BFF2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52BFF2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812BFF2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52BFF2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812BFF2B2BE3E32BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52BFF2B2BD8D82BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B818181812B2B2B2B2BFA2BFF000000000000000000"
	$"000000000000000000FF2BF5F5F5F52B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FF2B81FA81FA2B2B2B2B2BFA2BFF000000000000000000"
	$"000000000000000000FF2B2B2B2B2B2B2B2B2B2B2B2BFF000000000000000000"
	$"000000000000000000FCFFFFFFFFFFFFFFFFFFFFFFFFFC000000000000000000"
	};

resource 'icl4' (Quadra700IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"000000000EFFFFFFFFFFFFE000000000"
	$"000000000FCCCCCCCCCCCCF000000000"
	$"000000000FCCCCCCCCCCCCF000000000"
	$"000000000FCDDDDCFCCC1CF000000000"
	$"000000000FCC0C0CFCCC1CF000000000"
	$"000000000FCDDDDCFCCCCCF000000000"
	$"000000000FCC0C0CFCCCCCF000000000"
	$"000000000FCDDDDCFCCCCCF000000000"
	$"000000000FCC0C0CFCCCCCF000000000"
	$"000000000FCDDDDCFCC88CF000000000"
	$"000000000FCC0C0CFCC33CF000000000"
	$"000000000FCDDDDCCCCCCCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCCCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCCCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCCCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCCCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCCCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCDCF000000000"
	$"000000000FCC0C0CCCCCCCF000000000"
	$"000000000FCDDDDCCCCCDCF000000000"
	$"000000000FCCCCCCCCCCCCF000000000"
	$"000000000EFFFFFFFFFFFFE000000000"
	};

resource 'ics8' (Quadra700IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000FCFFFFFFFFFFFFFC00000000"
	$"00000000FF2B2B2B2B2B2BFF00000000"
	$"00000000FFF5F5F5FF2B2BFF00000000"
	$"00000000FF2B2B2BFF2B2BFF00000000"
	$"00000000FFF5F5F5FF2B2BFF00000000"
	$"00000000FF2B2B2BFFE3E3FF00000000"
	$"00000000FFF5F5F52B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2BFF00000000"
	$"00000000FFF5F5F52B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2BFF00000000"
	$"00000000FFF5F5F52B2B2BFF00000000"
	$"00000000FF2B2B2B2BFA2BFF00000000"
	$"00000000FFF5F5F52BFA2BFF00000000"
	$"00000000FF2B2B2B2B2B2BFF00000000"
	$"00000000FCFFFFFFFFFFFFFC00000000"
	};
	
resource 'ics4' (Quadra700IconResource, purgeable)
	{
	$"0000000000000000"
	$"0000EFFFFFFE0000"
	$"0000FCCCCCCF0000"
	$"0000FDDDFCCF0000"
	$"0000FCCCFCCF0000"
	$"0000FDDDFCCF0000"
	$"0000FCCCFCCF0000"
	$"0000FDDDCCCF0000"
	$"0000FCCCCCCF0000"
	$"0000FDDDCCCF0000"
	$"0000FCCCCCCF0000"
	$"0000FDDDCCCF0000"
	$"0000FCCCCDCF0000"
	$"0000FDDDCDCF0000"
	$"0000FCCCCCCF0000"
	$"0000EFFFFFFE0000"
	};

//__________________________________________________________________________________________________
// Quadra 900

resource 'ICN#' (Quadra900IconResource, purgeable)
	{ {
	$"00000000 00FFFF00 00800100 008FFF00"
	$"00880100 00880100 008BFD00 00880100"
	$"00880100 008FFF00 00880100 008A0100"
	$"00880100 00880100 00880100 00880100"
	$"00880100 00880100 00880100 00880100"
	$"00880100 008BFD00 00880100 008BFD00"
	$"00880100 008BFD00 00880100 008BFD00"
	$"00880100 008BFD00 00880100 00FFFF00";

	$"00000000 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	$"00FFFF00 00FFFF00 00FFFF00 00FFFF00"
	} };

resource 'ics#' (Quadra900IconResource, purgeable)
	{ {
	$"0FF0 0BF0 0A10 0AF0 0A10 0A10 0A10 0A10"
	$"0A10 0A10 0BF0 0A10 0BF0 0A10 0BF0 0FF0";

	$"0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0"
	$"0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0 0FF0"
	} };

resource 'icl8' (Quadra900IconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000"
	$"0000000000000000FFF8F8F8F8F8F8F8F8F8F8F8F8F8F8FF0000000000000000"
	$"0000000000000000FFF805F8FFFFFFFFFFFFFFFFFFFFFFFF0000000000000000"
	$"0000000000000000FFF805F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FFF7FFFFFFFFFFFFFFFFF7FF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FFFBFBFBFBFBFBFBFBFBFBFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BE3E32B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BD8D82B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF9F9F9F9F9F9F9F92BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF5F5F5F5F5F5F5F52BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF9F9F9F9F9F9F9F92BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF5F5F5F5F5F5F5F52BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF9F9F9F9F9F9F9F92BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF5F5F5F5F5F5F5F52BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF9F9F9F9F9F9F9F92BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF5F5F5F5F5F5F5F52BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2BF9F9F9F9F9F9F9F92BFF0000000000000000"
	$"0000000000000000FFF8F8F8FF2B2B2B2B2B2B2B2B2B2BFF0000000000000000"
	$"0000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000000000"
	};

resource 'icl4' (Quadra900IconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000FFFFFFFFFFFFFFFF00000000"
	$"00000000FCCCCCCCCCCCCCCF00000000"
	$"00000000FC1CFFFFFFFFFFFF00000000"
	$"00000000FC1CFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCFFFFFFFFCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFDDDDDDDDDDF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFC88CCCCCCCF00000000"
	$"00000000FCCCFC33CCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FCCCFCDDDDDDDDCF00000000"
	$"00000000FCCCFC0C0C0C0CCF00000000"
	$"00000000FCCCFCDDDDDDDDCF00000000"
	$"00000000FCCCFC0C0C0C0CCF00000000"
	$"00000000FCCCFCDDDDDDDDCF00000000"
	$"00000000FCCCFC0C0C0C0CCF00000000"
	$"00000000FCCCFCDDDDDDDDCF00000000"
	$"00000000FCCCFC0C0C0C0CCF00000000"
	$"00000000FCCCFCDDDDDDDDCF00000000"
	$"00000000FCCCFCCCCCCCCCCF00000000"
	$"00000000FFFFFFFFFFFFFFFF00000000"
	};

resource 'ics8' (Quadra900IconResource, purgeable)
	{
	$"00000000FFFFFFFFFFFFFFFF00000000"
	$"00000000FFF8FF2B2B2B2BFF00000000"
	$"00000000FFF8FFFFFFFFFFFF00000000"
	$"00000000FFF8FF2B2B2B2BFF00000000"
	$"00000000FFF8FFE32B2B2BFF00000000"
	$"00000000FFF8FFD82B2B2BFF00000000"
	$"00000000FFF8FF2B2B2B2BFF00000000"
	$"00000000FFF8FF2B2B2B2BFF00000000"
	$"00000000FFF8FF2B2B2B2BFF00000000"
	$"00000000FFF8FF2B2B2B2BFF00000000"
	$"00000000FFF8FFF9F9F9F9FF00000000"
	$"00000000FFF8FFF9F9F9F9FF00000000"
	$"00000000FFF8FFF9F9F9F9FF00000000"
	$"00000000FFF8FFF9F9F9F9FF00000000"
	$"00000000FFF8FFF9F9F9F9FF00000000"
	$"00000000FFFFFFFFFFFFFFFF00000000"
	};

resource 'ics4' (Quadra900IconResource, purgeable)
	{
	$"0000FFFFFFFF0000"
	$"0000FCFCCCCF0000"
	$"0000FCFFFFFF0000"
	$"0000FCFCCCCF0000"
	$"0000FCF8CCCF0000"
	$"0000FCF3CCCF0000"
	$"0000FCFCCCCF0000"
	$"0000FCFCCCCF0000"
	$"0000FCFCCCCF0000"
	$"0000FCFCCCCF0000"
	$"0000FCFDDDDF0000"
	$"0000FCFDDDDF0000"
	$"0000FCFDDDDF0000"
	$"0000FCFDDDDF0000"
	$"0000FCFDDDDF0000"
	$"0000FFFFFFFF0000"
	};

//__________________________________________________________________________________________________
// icons from system bundle
//__________________________________________________________________________________________________

//__________________________________________________________________________________________________
// generic Macintosh

resource 'ICN#' (MacUnknownIconResource, purgeable)
	{ {
	$"00000000 07FFFFE0 08000010 08000010"
	$"08FFFF10 09000090 09555090 09000090"
	$"09550090 09000090 09500090 09000090"
	$"09500090 09000090 09540090 09000090"
	$"09000090 08FFFF10 08000010 08000010"
	$"08000010 08000010 0800FF10 08000010"
	$"08000010 08000010 08000010 07FFFFE0"
	$"04000020 04000020 04000020 07FFFFE0";

	$"00000000 07FFFFE0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 0FFFFFF0"
	$"0FFFFFF0 0FFFFFF0 0FFFFFF0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	} };

resource 'ics#' (MacUnknownIconResource, purgeable)
	{ {
	$"0000 1FF8 2004 2FF4 2814 2A94 2814 2A14"
	$"2814 2FF4 2004 2004 20F4 2004 1FF8 1FF8";

	$"0000 1FF8 3FFC 3FFC 3FFC 3FFC 3FFC 3FFC"
	$"3FFC 3FFC 3FFC 3FFC 3FFC 3FFC 1FF8 1FF8"
	} };

resource 'icl8' (MacUnknownIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2BFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC2B2B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2BFC2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A002B2BFF00000000"
	$"00000000FF2B2B2B000000000000000000000000000000002B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2BFFFFFFFFFFFFFFFF2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B00000000000000002B2B2BFF00000000"
	$"00000000FF2B2BE3E32B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2BD8D82B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"00000000FF2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2BFF00000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	$"0000000000FFFBFBFBFBFBFBFCFCFCFCFCFDFDFDFDFDFDFDFDFDFF0000000000"
	$"0000000000FFFAFAFAFAFAFAFAFAFBFBFCFCFCFCFDFDFDFDFDFDFF0000000000"
	$"0000000000FFF9F9F9F9F9F9F9F9FAFAFAFBFBFCFCFCFCFDFDFDFF0000000000"
	$"0000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000000000"
	};

resource 'icl4' (MacUnknownIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCEEEEEEEEEEEEEEEECCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCE0D0D0D0D0D0D0D0D0CCF0000"
	$"0000FCCED0D0D0D0D0D0D0D0CCCF0000"
	$"0000FCCC0C0C0C0C0C0C0C0C0CCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCFFFFFFFFCCCF0000"
	$"0000FCCCCCCCCCCC0C0C0C0CCCCF0000"
	$"0000FCC88CCCCCCCCCCCCCCCCCCF0000"
	$"0000FCC33CCCCCCCCCCCCCCCCCCF0000"
	$"0000FCCCCCCCCCCCCCCCCCCCCCCF0000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	$"00000FEEEEEEEEEEEEFFFFFFFFF00000"
	$"00000FDDDDDDDDEEEEEEEEFFFFF00000"
	$"00000FDDDDDDDDDDDEEEEEEEEEF00000"
	$"00000FFFFFFFFFFFFFFFFFFFFFF00000"
	};

resource 'ics8' (MacUnknownIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BFCFCFCFCFCFCFCFC2BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFC2A2A2A2A2A2A002BFF0000"
	$"0000FF2BFA000000000000002BFF0000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BE32B2B2B2B2B2B2B2BFF0000"
	$"0000FF2BD82B2B2BFFFFFFFF2BFF0000"
	$"0000FF2B2B2B2B2B2B2B2B2B2BFF0000"
	$"000000FDFDFDFDFDFDFFFFFFFF000000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'ics4' (MacUnknownIconResource, purgeable)
	{
	$"0000000000000000"
	$"000FFFFFFFFFF000"
	$"00FCCCCCCCCCCF00"
	$"00FCEEEEEEEECF00"
	$"00FCED0D0D0CCF00"
	$"00FCE0D0D0D0CF00"
	$"00FCED0D0D0CCF00"
	$"00FCE0D0D0D0CF00"
	$"00FCED0D0D0CCF00"
	$"00FCD0C0C0C0CF00"
	$"00FCCCCCCCCCCF00"
	$"00FC8CCCCCCCCF00"
	$"00FC3CCCFFFFCF00"
	$"00FCCCCCCCCCCF00"
	$"000FFFFFFFFFF000"
	$"000FFFFFFFFFF000"
	};

resource 'SICN' (MacUnknownIconResource, purgeable)
	{ {
	$"0000 0000 1FF0 2008 2FE8 2828 2828 2828"
	$"2FE8 2008 21E8 2008 1FF0 1010 1FF0 0000"
	} };

//__________________________________________________________________________________________________
// suitcase-like icon for the System file

resource 'ICN#' (systemSuitcaseIconResource, purgeable)
	{ {
	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 0003F000 00040800"
	$"0FFBF7FE 100A1403 200E1C05 40000009"
	$"7FFFFFF1 80000011 807FE011 80801011"
	$"80BFD011 80A05011 80AA5011 80A05011"
	$"80A85011 80A05011 80BFD011 80801011"
	$"80801011 8083D011 80801011 807FE011"
	$"807FE012 80000014 80000018 7FFFFFF0";

	$"00000000 00000000 00000000 00000000"
	$"00000000 00000000 0003F000 0007F800"
	$"0FFFFFFE 1FFFFFFF 3FFFFFFF 7FFFFFFF"
	$"7FFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF"
	$"FFFFFFFE FFFFFFFC FFFFFFF8 7FFFFFF0"
	} };

resource 'ics#' (systemSuitcaseIconResource, purgeable)
	{ {
	$"0000 03C0 0420 3BDF 4421 FFFD 8005 8FC5"
	$"8845 8B45 8A45 8845 8845 8785 8006 FFFC";

	$"0000 03C0 07E0 3FFF 7FFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFE FFFC"
	} };

resource 'icl8' (systemSuitcaseIconResource, purgeable)
	{
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000000000000000000000000000000000000000"
	$"0000000000000000000000000000FFFFFFFFFFFF000000000000000000000000"
	$"00000000000000000000000000FFF7F7F7F7F7F7FF0000000000000000000000"
	$"00000000FFFFFFFFFFFFFFFFFFF7FBFBFBFBFBFBF7FFFFFFFFFFFFFFFFFFFF00"
	$"000000FFF5F5F5F5F5F5F5F5FBF7FBF5F5F5F5FBF7FBF5F5F5F5F5F5F5F5FFFF"
	$"0000FFF5F5F5F5F5F5F5F5F5FBFBFBF5F5F5F5FBFBFBF5F5F5F5F5F5F5FBF7FF"
	$"00FFF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5FBF7F7FF"
	$"00FFFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6F6FFFFFFFFFFFFFFFFFFFFF6F6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2B2B2B2B2B2B2B2B2B2BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFCFCFCFCFCFCFCFC2BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFC2A2A2A2A2A2A002BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFC2A2A2A2A2A2A002BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFC2A2A2A2A2A2A002BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFC2A2A2A2A2A2A002BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFC2A2A2A2A2A2A002BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BFA000000000000002BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2B2B2B2B2B2B2B2B2B2BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BE32B2B2B2B2B2B2B2BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2BD82B2B2BFFFFFFFF2BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6FF2B2B2B2B2B2B2B2B2B2BFFF6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6F6FFFDFDFDFDFDFDFDFDFFF6F6F6F6F6F6F6F6FBF7F7F7FF"
	$"FFF6F6F6F6F6F6F6F6FFFFFFFFFFFFFFFFFFFFF6F6F6F6F6F6F6F6FBF7F7FF00"
	$"FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FBF7FF0000"
	$"FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FBFF000000"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	};

resource 'icl4' (systemSuitcaseIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000000000000000000000"
	$"00000000000000FFFFFF000000000000"
	$"0000000000000FCCCCCCF00000000000"
	$"0000FFFFFFFFFCEEEEEECFFFFFFFFFF0"
	$"000FC0C0C0C0ECE0C0CECEC0C0C0C0FF"
	$"00FC0C0C0C0CEEEC0C0EEE0C0C0C0EDF"
	$"0FC0C0C0C0C0C0C0C0C0C0C0C0C0EDDF"
	$"0FEEEEEEEEEEEEEEEEEEEEEEEEEEDDDF"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0CEDDDF"
	$"FC0C0C0C0FFFFFFFFFF00C0C0C0EDDDF"
	$"F0C0C0C0FCCCCCCCCCCFC0C0C0CEDDDF"
	$"FC0C0C0CFCEEEEEEEECF0C0C0C0EDDDF"
	$"F0C0C0C0FCED0D0D0CCFC0C0C0CEDDDF"
	$"FC0C0C0CFCE0D0D0D0CF0C0C0C0EDDDF"
	$"F0C0C0C0FCED0D0D0CCFC0C0C0CEDDDF"
	$"FC0C0C0CFCE0D0D0D0CF0C0C0C0EDDDF"
	$"F0C0C0C0FCED0D0D0CCFC0C0C0CEDDDF"
	$"FC0C0C0CFCD0C0C0C0CF0C0C0C0EDDDF"
	$"F0C0C0C0FCCCCCCCCCCFC0C0C0CEDDDF"
	$"FC0C0C0CFC8CCCCCCCCF0C0C0C0EDDDF"
	$"F0C0C0C0FC3CCCFFFFCFC0C0C0CEDDDF"
	$"FC0C0C0CFCCCCCCCCCCF0C0C0C0EDDDF"
	$"F0C0C0C0CFFFFFFFFFF0C0C0C0CEDDDF"
	$"FC0C0C0C0FFFFFFFFFFC0C0C0C0EDDF0"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0CEDF00"
	$"FC0C0C0C0C0C0C0C0C0C0C0C0C0EF000"
	$"0FFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics8' (systemSuitcaseIconResource, purgeable)
	{
	$"00000000000000000000000000000000"
	$"000000000000FEFEFEFE000000000000"
	$"0000000000FEF7F7F7F7FE0000000000"
	$"0000FFFFFFF7FEFFFFFFF7FEFFFFFFFF"
	$"00FFF6F6F6FEF6F6F6F6FEF6F6F6F7FF"
	$"FFFCFCFCFCFCFCFCFCFCFCFCFCFCF7FF"
	$"FFF5F5F5F5F5F5F5F5F5F5F5F5FCF7FF"
	$"FFF5F5F5FFFFFFFFFFFFF5F500FCF7FF"
	$"FFF5F5F5FF2B2B2B2BFFF5F5F5FCF7FF"
	$"FFF5F5F5FF2BFCFCF5FFF5F5F5FCF7FF"
	$"FFF5F5F5FF2BFC2AF5FFF5F5F5FCF7FF"
	$"FFF5F5F5FF2BF5F5F5FFF5F5F5FCF7FF"
	$"FFF5F5F5FF2B2B2B2BFFF5F5F5FCF7FF"
	$"FFF5F5F5F5FFFFFFFFF5F5F5F5FCF7FF"
	$"FFF5F5F5F5F5F5F5F5F5F5F5F5FCFF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics4' (systemSuitcaseIconResource, purgeable)
	{
	$"0000000000000000"
	$"000000FFFF000000"
	$"00000FCCCCF00000"
	$"00FFFCFFFFCFFFFF"
	$"0F0C0F0C0CFC0CDF"
	$"FEEEEEEEEEEEEEDF"
	$"F0C0C0C0C0C0CEDF"
	$"FC0CFFFFFF0C0EDF"
	$"F0C0FCCCCFC0CEDF"
	$"FC0CFCEE0F0C0EDF"
	$"F0C0FCEC0FC0CEDF"
	$"FC0CFC000F0C0EDF"
	$"F0C0FCCCCFC0CEDF"
	$"FC0C0FFFFC0C0EDF"
	$"F0C0C0C0C0C0CEF0"
	$"FFFFFFFFFFFFFF00"
	};

//__________________________________________________________________________________________________
// generic extension

resource 'ICN#' (genericExtensionIconResource, purgeable)
	{ {
	$"7FFFFFF0 80000008 80000008 80000008"
	$"80000008 80000008 80000008 80000008"
	$"A0000008 D000000A 9000000D 10000009"
	$"10000001 10000001 10000001 10000001"
	$"10000001 10000001 10000001 10000001"
	$"10000009 9000000D D000000A A0000008"
	$"80000008 80000008 80000008 80000008"
	$"80000008 80000008 80000008 7FFFFFF0";

	$"7FFFFFF0 FFFFFFF8 FFFFFFF8 FFFFFFF8"
	$"FFFFFFF8 FFFFFFF8 FFFFFFF8 FFFFFFF8"
	$"FFFFFFF8 DFFFFFFA 9FFFFFFF 1FFFFFFF"
	$"1FFFFFFF 1FFFFFFF 1FFFFFFF 1FFFFFFF"
	$"1FFFFFFF 1FFFFFFF 1FFFFFFF 1FFFFFFF"
	$"1FFFFFFF 9FFFFFFF DFFFFFFA FFFFFFF8"
	$"FFFFFFF8 FFFFFFF8 FFFFFFF8 FFFFFFF8"
	$"FFFFFFF8 FFFFFFF8 FFFFFFF8 7FFFFFF0"
	} };

resource 'ics#' (genericExtensionIconResource, purgeable)
	{ {
	$"FFFE 8002 8002 8002 8002 C003 4001 4001"
	$"4001 4001 C003 8002 8002 8002 8002 FFFE";

	$"FFFE FFFE FFFE FFFE FFFE FFFF 7FFF 7FFF"
	$"7FFF 7FFF 7FFF FFFE FFFE FFFE FFFE 7FFC"
	} };

resource 'icl8' (genericExtensionIconResource, purgeable)
	{
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	$"FF000000000000000000000000000000000000000000000000000000FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF0000F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FFFF00FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF00FF00"
	$"FF0000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FFFF00FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF00F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"000000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FFF6F6FF"
	$"FF0000FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FFFFF6FF"
	$"FFFF00FFF6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF00FF00"
	$"FF00FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6FF000000"
	$"00FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00000000"
	};

resource 'icl4' (genericExtensionIconResource, purgeable)
	{
	$"0FFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	$"F000000000000000000000000000F000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0F0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"FF0F0C0C0C0C0C0C0C0C0C0C0C0CF0F0"
	$"F00FC0C0C0C0C0C0C0C0C0C0C0C0FF0F"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0CF00F"
	$"000FC0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0C0C0F"
	$"000FC0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0C0C0F"
	$"000FC0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0C0C0F"
	$"000FC0C0C0C0C0C0C0C0C0C0C0C0C0CF"
	$"000F0C0C0C0C0C0C0C0C0C0C0C0C0C0F"
	$"000FC0C0C0C0C0C0C0C0C0C0C0C0F0CF"
	$"F00F0C0C0C0C0C0C0C0C0C0C0C0CFF0F"
	$"FF0FC0C0C0C0C0C0C0C0C0C0C0C0F0F0"
	$"F0F00C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"F00C0C0C0C0C0C0C0C0C0C0C0C0CF000"
	$"F0C0C0C0C0C0C0C0C0C0C0C0C0C0F000"
	$"0FFFFFFFFFFFFFFFFFFFFFFFFFFF0000"
	};

resource 'ics8' (genericExtensionIconResource, purgeable)
	{
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	$"FF00000000000000000000000000FF00"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FFFF00F6F6F6F6F6F6F6F6F6F6F6FFFF"
	$"00FF00F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"00FF00F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"00FF00F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"00FF00F6F6F6F6F6F6F6F6F6F6F6F6FF"
	$"FFFFF6F6F6F6F6F6F6F6F6F6F6F6FFFF"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FF00F6F6F6F6F6F6F6F6F6F6F6F6FF00"
	$"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF00"
	};

resource 'ics4' (genericExtensionIconResource, purgeable)
	{
	$"FFFFFFFFFFFFFFF0"
	$"F0000000000000F0"
	$"F0C0C0C0C0C0C0F0"
	$"F00C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0F0"
	$"FF0C0C0C0C0C0CFF"
	$"0F00C0C0C0C0C0CF"
	$"0F0C0C0C0C0C0C0F"
	$"0F00C0C0C0C0C0CF"
	$"0F0C0C0C0C0C0C0F"
	$"FFC0C0C0C0C0C0FF"
	$"F00C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0F0"
	$"F00C0C0C0C0C0CF0"
	$"F0C0C0C0C0C0C0F0"
	$"FFFFFFFFFFFFFFF0"
	};

//__________________________________________________________________________________________________
// Clipboard File

resource 'ICN#' (clipboardFileIconResource, purgeable)
	{ {
	$"0003C000 00042000 0005A000 0005A000"
	$"03F81FC0 06200460 04400220 047FFE20"
	$"04040420 04080220 04100120 042000A0"
	$"04400120 04800220 05000420 04800820"
	$"04401020 04202020 04104020 04088020"
	$"04050020 04020020 04000020 04000020"
	$"04000020 04000020 04000020 04000020"
	$"04000020 04000020 06000060 03FFFFC0";

	$"0003C000 0007E000 0007E000 0007E000"
	$"03FFFFC0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 07FFFFE0"
	$"07FFFFE0 07FFFFE0 07FFFFE0 03FFFFC0"
	} };

resource 'ics#' (clipboardFileIconResource, purgeable)
	{ {
	$"0380 0440 1D70 2828 2FE8 2288 2448 2888"
	$"2508 2208 2008 2008 2008 2008 2008 1FF0";

	$"0380 07C0 1FF0 3FF8 3FF8 3FF8 3FF8 3FF8"
	$"3FF8 3FF8 3FF8 3FF8 3FF8 3FF8 3FF8 1FF0"
	} };

resource 'icl8' (clipboardFileIconResource, purgeable)
	{
	$"0000000000000000000000000000FFFFFFFF0000000000000000000000000000"
	$"00000000000000000000000000FFF500F500FF00000000000000000000000000"
	$"00000000000000000000000000FF00FFFFF5FF00000000000000000000000000"
	$"00000000000000000000000000FFF5FFFF00FF00000000000000000000000000"
	$"000000000000FFFFFFFFFFFFFFF500F500F500FFFFFFFFFFFFFF000000000000"
	$"0000000000FFFFF733F7FA00F500F500F500F500F5FF33F733FFFF0000000000"
	$"0000000000FFF733F7FDF8F8F8F8F8F8F8F8F8F8F8F8FD33F733FF0000000000"
	$"0000000000FF33F733FDFDFDFDFDFDFDFDFDFDFDFDFDFDF733F7FF0000000000"
	$"0000000000FFF733F733F733F7FD00000000000000FDF733F733FF0000000000"
	$"0000000000FF33F733F733F7FF000000000000000000FFF733F7FF0000000000"
	$"0000000000FFF733F733F7FF0000000000000000000000FFF733FF0000000000"
	$"0000000000FF33F733F7FF00000000000000000000000000FFF7FF0000000000"
	$"0000000000FFF733F7FF00000000000000000000000000FFF733FF0000000000"
	$"0000000000FF33F7FF00000000000000000000000000FFF733F7FF0000000000"
	$"0000000000FFF7FF00000000000000000000000000FFF733F733FF0000000000"
	$"0000000000FF33F7FF0000000000000000000000FFF733F733F7FF0000000000"
	$"0000000000FFF733F7FF000000000000000000FFF733F733F733FF0000000000"
	$"0000000000FF33F733F7FF00000000000000FFF733F733F733F7FF0000000000"
	$"0000000000FFF733F733F7FF0000000000FFF733F733F733F733FF0000000000"
	$"0000000000FF33F733F733F7FF000000FFF733F733F733F733F7FF0000000000"
	$"0000000000FFF733F733F733F7FF00FFF733F733F733F733F733FF0000000000"
	$"0000000000FF33F733F733F733F7FFF733F733F733F733F733F7FF0000000000"
	$"0000000000FFF733F733F733F733F733F733F733F733F733F733FF0000000000"
	$"0000000000FF33F733F733F733F733F733F733F733F733F733F7FF0000000000"
	$"0000000000FFF733F733F733F733F733F733F733F733F733F733FF0000000000"
	$"0000000000FF33F733F733F733F733F733F733F733F733F733F7FF0000000000"
	$"0000000000FFF733F733F733F733F733F733F733F733F733F733FF0000000000"
	$"0000000000FF33F733F733F733F733F733F733F733F733F733F7FF0000000000"
	$"0000000000FFF733F733F733F733F733F733F733F733F733F733FF0000000000"
	$"0000000000FF33F733F733F733F733F733F733F733F733F733F7FF0000000000"
	$"0000000000FFFF33F733F733F733F733F733F733F733F733F7FFFF0000000000"
	$"000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000"
	};

resource 'icl4' (clipboardFileIconResource, purgeable)
	{
	$"00000000000000FFFF00000000000000"
	$"0000000000000F0000F0000000000000"
	$"0000000000000F0FF0F0000000000000"
	$"0000000000000F0FF0F0000000000000"
	$"000000FFFFFFF000000FFFFFFF000000"
	$"00000FFCDCD0000000000FDCDFF00000"
	$"00000FCDCECCCCCCCCCCCCEDCDF00000"
	$"00000FDCDEEEEEEEEEEEEEECDCF00000"
	$"00000FCDCDCDCE0000000ECDCDF00000"
	$"00000FDCDCDCF000000000FCDCF00000"
	$"00000FCDCDCF00000000000FCDF00000"
	$"00000FDCDCF0000000000000FCF00000"
	$"00000FCDCF0000000000000FCDF00000"
	$"00000FDCF0000000000000FCDCF00000"
	$"00000FCF0000000000000FCDCDF00000"
	$"00000FDCF00000000000FCDCDCF00000"
	$"00000FCDCF000000000FCDCDCDF00000"
	$"00000FDCDCF0000000FCDCDCDCF00000"
	$"00000FCDCDCF00000FCDCDCDCDF00000"
	$"00000FDCDCDCF000FCDCDCDCDCF00000"
	$"00000FCDCDCDCF0FCDCDCDCDCDF00000"
	$"00000FDCDCDCDCFCDCDCDCDCDCF00000"
	$"00000FCDCDCDCDCDCDCDCDCDCDF00000"
	$"00000FDCDCDCDCDCDCDCDCDCDCF00000"
	$"00000FCDCDCDCDCDCDCDCDCDCDF00000"
	$"00000FDCDCDCDCDCDCDCDCDCDCF00000"
	$"00000FCDCDCDCDCDCDCDCDCDCDF00000"
	$"00000FDCDCDCDCDCDCDCDCDCDCF00000"
	$"00000FCDCDCDCDCDCDCDCDCDCDF00000"
	$"00000FDCDCDCDCDCDCDCDCDCDCF00000"
	$"00000FFDCDCDCDCDCDCDCDCDCFF00000"
	$"000000FFFFFFFFFFFFFFFFFFFF000000"
	};

resource 'ics8' (clipboardFileIconResource, purgeable)
	{
	$"000000000000FFFFFF00000000000000"
	$"0000000000FF00F600FF000000000000"
	$"000000FFFFFFF6FFF6FFFFFF00000000"
	$"0000FF33FAF600F600F6FD33FF000000"
	$"0000FFF7FDFDFDFDFDFDFDF7FF000000"
	$"0000FF33F733FF00FF33F733FF000000"
	$"0000FFF733FF000000FF33F7FF000000"
	$"0000FF33FF000000FF33F733FF000000"
	$"0000FFF733FF00FF33F733F7FF000000"
	$"0000FF33F733FF33F733F733FF000000"
	$"0000FFF733F733F733F733F7FF000000"
	$"0000FF33F733F733F733F733FF000000"
	$"0000FFF733F733F733F733F7FF000000"
	$"0000FF33F733F733F733F733FF000000"
	$"0000FFF733F733F733F733F7FF000000"
	$"000000FFFFFFFFFFFFFFFFFF00000000"
	};

resource 'ics4' (clipboardFileIconResource, purgeable)
	{
	$"000000FFF0000000"
	$"00000F000F000000"
	$"000FFF0F0FFF0000"
	$"00FCE00000FCF000"
	$"00FDFFFFFFFDF000"
	$"00FCDCF0FCDCF000"
	$"00FDCF000FCDF000"
	$"00FCF000FCDCF000"
	$"00FDCF0FCDCDF000"
	$"00FCDCFCDCDCF000"
	$"00FDCDCDCDCDF000"
	$"00FCDCDCDCDCF000"
	$"00FDCDCDCDCDF000"
	$"00FCDCDCDCDCF000"
	$"00FDCDCDCDCDF000"
	$"000FFFFFFFFF0000"
	};

//__________________________________________________________________________________________________

